<?php
/**
 * PDEL for PHP5
 * (Practical and Decryptable Encryption Library for PHP5)
 * http://pdel.vpsdemo.biz/
 *
 * @category   exceptionCatcher's Library
 * @copyright  exceptionCatcher(http://exceptioncatcher.seesaa.net/)
 * @license    MIT License
 * @version    1.0
 */
class Pdel
{
    //特殊文字1(16進数マッピング用1)
    const SPECIAL_CHAR1 = '.';
    //特殊文字2(16進数マッピング用2)
    const SPECIAL_CHAR2 = '-';
    //マップファイル先頭固定部
    const MAP_FILE_PRE = '.MAP-';
    //マップファイル拡張子
    const MAP_FILE_EXT = '.txt';
    //レシピファイル先頭固定部
    const RECIPE_FILE_PRE = '.RECIPE-';
    //レシピファイル拡張子
    const RECIPE_FILE_EXT = '.txt';
    //レシピ名先頭1桁(固定)
    const RECIPE_PTN_F = 'F';
    //レシピ名先頭1桁(可変)
    const RECIPE_PTN_V = 'V';
    //レシピ名最大桁数
    const RECIPE_MAX_LEN = 40;
    //MASK名最大桁数
    const MASK_MAX_LEN = 40;
    //固定変換用マップファイルサブスクリプト情報最大桁数
    const FIXED_MAP_SS_MAX = 16;
    //暗号化最大バイト数
    const ENCRYPT_MAX_SIZE = 10000;
    //復号化最大バイト数(10000バイト暗号化+認証情報Aを認証モード7+Addコマンドで100バイト追加×100回)
    const DECRYPT_MAX_SIZE = 30044;
    //Extend最小値
    const EXTEND_MIN = 50;
    //Extend最大値
    const EXTEND_MAX = 2000;
    //LimitTime最大値(最大30日)
    const LIMIT_MAX = 2592000;
    //Add最大値
    const ADD_MAX = 100;
    //レシピコマンド指定最大件数
    const RECIPE_CMD_MAXF = 100;
    //MAP文字数
    const MAP_CHAR_NUM = 64;
    //セッションハイジャックチェックキー項目文字列最大長
    const SESSION_HIJACK_CHECK_KEY_MAX_LEN = 65535;
    //マップファイル・レシピファイルで実施するチェック項目
    private $_i_mandatory_file_check_type;
    //マスクルールファイルで実施するチェック項目
    private $_i_mask_rule_file_check_type;
    //復号スキップフラグ
    private $_b_skip_decrypt;
    //STRICTマスクフラグ
    private $_b_strict_mask;
    //マスク区分(0:マスクなし　1:常にマスク　2:権限に応じたマスク<NUM型>　3:権限に応じたマスク<NUM_STR型>　4:権限に応じたマスク<STR型>)
    private $_i_mask_type;
    //マスクユーザー数値権限最小値
    private $_i_mask_user_level_min;
    //マスクユーザー数値権限最大値
    private $_i_mask_user_level_max;
    //ユーザー権限レベル情報
    private $_s_user_level_list;
    //ユーザー権限レベル情報(Empty値動作定義)
    private $_a_user_level_empty;
    //ユーザー権限情報昇順区分
    private $_b_user_level_asc_order;
    //文字列型ユーザー権限情報区切り文字
    private $_s_user_level_list_delimiter;
    //処理結果
    private $_s_result;
    //MAP情報
    private $_s_map_str;
    private $_s_extend_str;
    private $_s_map_fixed_ss;
    //レシピ情報
    private $_i_recipe_max;
    private $_a_recipe_inf;
    //固定モード
    private $_b_fixed_mode;
    //処理対象レシピ配列インデックス
    private $_i_current_recipe;
    //復号モード
    private $_b_decrypt_mode;
    //バイナリモード
    private $_b_bin_mode;
    //ChangeAuth
    private $_s_target_recipe;
    //デバッグ情報
    private $_b_debug_mode;
    private $_a_debug_inf;
    private $_f_debug_start_time;
    private $_f_debug_last_time;
    //エラー情報
    private $_s_error_inf;
    //復号エラー区分
    private $_s_decrypt_err_category;
    //ASCII文字列
    private $_ASCII_STR;
    //定義ファイル格納ディレクトリ
    private $_s_mandatory_file_dir;
    //マスクルール定義ファイル格納ディレクトリ
    private $_s_mask_rule_file_dir;
    //マスクルールファイル
    private $_s_mask_rule_file;
    //変換コマンド開始行
    private $_i_cnv_cmd_start;
    //デフォルト認証タイプ
    private $_i_default_auth_mode;
    //ハイジャックチェック認証タイプ
    private $_i_auth_inf_c_mode;
    //認証情報レングス(認証モードごとのレングス)
    private $_a_auth_mode_len;
    private $_b_limit_by_encrypt_param;
    //セッションハイジャックチェック項目
    private $_s_hijack_check_ip;
    private $_b_hijack_check_ua;
    private $_b_hijack_check_lang;
    //復号時のユーザー権限レベル
    private $_m_param_user_level;
    private $_b_param_user_level_var;
    //マスクルールファイル情報
    private $_a_mask_rule_file;
    private $_i_mask_rule_index;
    //認証情報B
    private $_s_auth_inf_b;
    //ユーザー指定任意情報
    private $_s_user_key_c;
    //認証情報C
    private $_s_auth_inf_c;
    //DateTimeClass
    private $_b_date_time;
    //時差情報
    private $_i_time_difference;
    /**
     * コンストラクタ
     *
     * @param boolean $s_ini_file    PDEL初期化ファイル
     * @param boolean $b_debug_mode  デバッグモード
     * @return void
     */
    public function __construct($s_ini_file = '' , $b_debug_mode = false)
    {
        //コンストラクタ内で認識したエラー(パラメータミス、INIファイル定義ミス)はScriptを強制終了します。
        try{
            if( is_string($s_ini_file) === false ) throw new Exception('invalid ini file(not string)');
            if( strpos($s_ini_file , chr(0)) !== false ) throw new Exception('invalid ini file(null exists)');
            if( is_bool($b_debug_mode) === false ) throw new Exception('invalid debug mode');
            $this->_b_debug_mode = $b_debug_mode;
            //認証モード0～7の長さ
            $this->_a_auth_mode_len = array(0 , 4 , 8 , 32 , 40 , 12 , 36 , 44);
            $this->_s_error_inf = '';
            $this->_ASCII_STR = self::SPECIAL_CHAR1 . self::SPECIAL_CHAR2 .
                                '0123456789' .
                                'ABCDEFGHIJKLMNOPQRSTUVWXYZ' .
                                'abcdefghijklmnopqrstuvwxyz';
            //時差情報
            if( class_exists('DateTime') === true ){
                $this->_b_date_time = true;
                $o_local = new DateTime();
                $this->_i_time_difference = $o_local->getOffset();
            }
            else{
                $this->_b_date_time = false;
                $i_local_time = time();
                $s_gmt = gmdate('YmdHis');
                $i_gmt_time = mktime(intval(substr($s_gmt , 8 , 2)) , intval(substr($s_gmt , 10 , 2)) , substr($s_gmt , 12 , 2) ,
                                intval(substr($s_gmt , 4 , 2)) , intval(substr($s_gmt , 6 , 2)) , substr($s_gmt , 0 , 4) );
                $this->_i_time_difference = $i_local_time - $i_gmt_time;
                switch(true){
                    case abs($this->_i_time_difference) > (24 * 60 * 60):
                        //2038
                        $this->_i_time_difference = 0;
                        break;
                    case ($this->_i_time_difference % 10) !== 0:
                        //time()実行時とgmdate('YmdHis')実行時の秒が異なるケース
                        $this->_i_time_difference = $this->_i_time_difference + 1;
                        break;
                }
            }
            //INI解析
            $this->_parsePdelIniFile($s_ini_file);
        } catch (Exception $exc) {
            $this->_die(__FUNCTION__ . '(' . __LINE__ . ')' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage());
        }
    }
    /**
     * Pdel用INIファイル解析
     *
     * @param striing $s_file_name Pdel用INIファイル名
     * @return void
     */
    private function _parsePdelIniFile($s_ini_file)
    {
        /**
        * INIファイル名確定
        */
        if( $s_ini_file === '' ){
            $s_file_name = dirname(__FILE__);
            if( substr($s_file_name , -1) !== DIRECTORY_SEPARATOR ) $s_file_name = $s_file_name . DIRECTORY_SEPARATOR;
            $s_file_name = $s_file_name . '.pdel.ini';
            //ファイル存在チェック
            if(file_exists($s_file_name) === false ) throw new Exception($s_file_name . ' not exists');
            if(is_file($s_file_name) === false ) throw new Exception($s_file_name . ' not exists');
        }
        else{
            $s_file_name = $this->_parsePath($s_ini_file , true);
            if( $s_file_name === '' ) throw new Exception($s_ini_file . ' not exists');
        }
        //読み込み権限が存在することのチェック
        if( is_readable($s_file_name) === false ) throw new Exception($s_file_name . ' not readable');
        //INIファイル読み込み
        $a_ini_file = parse_ini_file($s_file_name , true);
        //parse_ini_fileの戻り値はPHPバージョンで異なるためemptyで判定する
        if( empty($a_ini_file) === true ) throw new Exception($s_file_name . ' parse error');
        if( array_key_exists('PDEL_ENV' , $a_ini_file) === false ) throw new Exception($s_file_name . ' PDEL_ENV not defined');
        $b_sect_mask_env_com = array_key_exists('MASK_ENV_COM' , $a_ini_file);
        $b_sect_mask_env_level_num = array_key_exists('MASK_ENV_LEVEL_NUM' , $a_ini_file);
        $b_sect_mask_env_level_num_str = array_key_exists('MASK_ENV_LEVEL_NUM_STR' , $a_ini_file);
        $b_sect_mask_env_level_str = array_key_exists('MASK_ENV_LEVEL_STR' , $a_ini_file);
        if( $b_sect_mask_env_com === true ){
            switch(true){
                case $b_sect_mask_env_level_num === false and $b_sect_mask_env_level_num_str === false and $b_sect_mask_env_level_str === false:
                    $this->_i_mask_type = 1;
                    break;
                case $b_sect_mask_env_level_num === true and $b_sect_mask_env_level_num_str === false and $b_sect_mask_env_level_str === false:
                    $this->_i_mask_type = 2;
                    break;
                case $b_sect_mask_env_level_num === false and $b_sect_mask_env_level_num_str === true and $b_sect_mask_env_level_str === false:
                    $this->_i_mask_type = 3;
                    break;
                case $b_sect_mask_env_level_num === false and $b_sect_mask_env_level_num_str === false and $b_sect_mask_env_level_str === true:
                    $this->_i_mask_type = 4;
                    break;
                default:
                    throw new Exception($s_file_name . ' MASK_ENV_LEVEL section def err');
            }
        }
        else{
            if( $b_sect_mask_env_level_num === true or $b_sect_mask_env_level_num_str === true or $b_sect_mask_env_level_str === true) throw new Exception($s_file_name . ' MASK_ENV_COM not defined');
            $this->_i_mask_type = 0;
        }
        $this->_s_mask_rule_file_dir = '';
        $this->_s_mask_rule_file = '';
        $this->_s_user_level_list = '';
        $this->_s_user_level_list_delimiter = '';
        $this->_a_user_level_empty = array();
        //INIファイルチェック用配列
        //第1フィールド：セクション情報
        //第2フィールド：キー名
        //第3フィールド：キー値属性情報(N:数値<最大値・最小値チェックあり> n:数値<最大値・最小値チェックなし> B:Boolean D:ディレクトリ S:文字列 C:1文字のキャラクタ
        //第4フィールド：キー値属性が数値の場合の最小値
        //第5フィールド：キー値属性が数値の場合の最大値
        $a_key_inf = array('PDEL_ENV:AUTH_MODE:N:0:7' ,
                           'PDEL_ENV:MFILE_DIR:D' ,
                           'PDEL_ENV:MFILE_CHK_TYPE:N:0:15' ,

                           'MASK_ENV_COM:OFILE_DIR:D' ,
                           'MASK_ENV_COM:OFILE:S' ,
                           'MASK_ENV_COM:OFILE_CHK_TYPE:N:0:15' ,
                           'MASK_ENV_COM:SKIP_DECRYPT:B' ,
                           'MASK_ENV_COM:STRICT_MASK:B' ,

                           'MASK_ENV_LEVEL_NUM:UL_MIN:n' ,
                           'MASK_ENV_LEVEL_NUM:UL_MAX:n' ,
                           'MASK_ENV_LEVEL_NUM:UL_ASC_ORDER:B' ,
                           'MASK_ENV_LEVEL_NUM:UL_EMPTY:S',

                           'MASK_ENV_LEVEL_NUM_STR:UL_LIST:S' ,
                           'MASK_ENV_LEVEL_NUM_STR:UL_LIST_DEL:C' ,
                           'MASK_ENV_LEVEL_NUM_STR:UL_ASC_ORDER:B' ,
                           'MASK_ENV_LEVEL_NUM_STR:UL_EMPTY:S' ,

                           'MASK_ENV_LEVEL_STR:UL_LIST:S' ,
                           'MASK_ENV_LEVEL_STR:UL_LIST_DEL:C' ,
                           'MASK_ENV_LEVEL_STR:UL_ASC_ORDER:B' ,
                           'MASK_ENV_LEVEL_STR:UL_EMPTY:S');

        for($i_ss1 = 0;$i_ss1 < count($a_key_inf);$i_ss1++){
            $a_tmp = explode(':' , $a_key_inf[$i_ss1]);
            if( ($a_tmp[0] === 'MASK_ENV_COM') and ($b_sect_mask_env_com === false) ) continue;
            if( ($a_tmp[0] === 'MASK_ENV_LEVEL_NUM') and ($b_sect_mask_env_level_num === false) ) continue;
            if( ($a_tmp[0] === 'MASK_ENV_LEVEL_NUM_STR') and ($b_sect_mask_env_level_num_str === false) ) continue;
            if( ($a_tmp[0] === 'MASK_ENV_LEVEL_STR') and ($b_sect_mask_env_level_str === false) ) continue;
            if( array_key_exists($a_tmp[1] , $a_ini_file[$a_tmp[0]]) === false ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' not defined');
            switch($a_tmp[2]){
                    case 'N':
                    case 'n':
                    if( ($a_tmp[1] === 'UL_MIN') and ($a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'PDEL-NULL') ){
                        $this->_i_mask_user_level_min = null;
                        break;
                    }
                    if( $a_tmp[1] === 'UL_MAX' ){
                        if( ($this->_i_mask_user_level_min === null) and ($a_ini_file[$a_tmp[0]][$a_tmp[1]] !== 'PDEL-NULL') ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                        if( ($this->_i_mask_user_level_min !== null) and ($a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'PDEL-NULL') ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                        if( $a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'PDEL-NULL' ){
                            $this->_i_mask_user_level_max = null;
                            break;
                        }
                    }
                    if( ctype_digit($a_ini_file[$a_tmp[0]][$a_tmp[1]]) === false ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    $i_num = intval($a_ini_file[$a_tmp[0]][$a_tmp[1]]);
                    if( ($a_tmp[2] === 'N') and ($i_num < intval($a_tmp[3]) or $i_num > intval($a_tmp[4])) ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    switch($a_tmp[1]){
                        case 'AUTH_MODE':
                            $this->_i_default_auth_mode = $i_num;
                            break;
                        case 'MFILE_CHK_TYPE':
                            $this->_i_mandatory_file_check_type = $i_num;
                            break;
                        case 'OFILE_CHK_TYPE':
                            $this->_i_mask_rule_file_check_type = $i_num;
                            break;
                        case 'UL_MIN':
                            $this->_i_mask_user_level_min = $i_num;
                            break;
                        case 'UL_MAX':
                            if( $i_num <= $this->_i_mask_user_level_min ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' must great UL_MIN');
                            $this->_i_mask_user_level_max = $i_num;
                            break;
                    }
                    break;
                case 'D':
                    $s_dir = $this->_parsePath($a_ini_file[$a_tmp[0]][$a_tmp[1]] , false);
                    if( $s_dir === '') throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    switch($a_tmp[1]){
                        case 'MFILE_DIR':
                            $this->_s_mandatory_file_dir = $s_dir;
                            break;
                        case 'OFILE_DIR':
                            $this->_s_mask_rule_file_dir = $s_dir;
                            break;
                    }
                    break;
                case 'B':
                    switch(true){
                        case $a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'true':
                            $b_val = true;
                            break;
                        case $a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'false':
                            $b_val = false;
                            break;
                        default:
                            throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    }
                    switch($a_tmp[1]){
                        case 'SKIP_DECRYPT':
                            $this->_b_skip_decrypt = $b_val;
                            break;
                        case 'STRICT_MASK':
                            $this->_b_strict_mask = $b_val;
                            break;
                        case 'UL_ASC_ORDER':
                            $this->_b_user_level_asc_order = $b_val;
                            break;
                    }
                    break;
                case 'S':
                    if( is_string($a_ini_file[$a_tmp[0]][$a_tmp[1]]) === false ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    if( $a_ini_file[$a_tmp[0]][$a_tmp[1]] === '' ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' must not null');
                    switch($a_tmp[1]){
                        case 'OFILE':
                            $this->_s_mask_rule_file = $a_ini_file[$a_tmp[0]][$a_tmp[1]];
                            break;
                        case 'UL_LIST':
                            $this->_s_user_level_list = $a_ini_file[$a_tmp[0]][$a_tmp[1]];
                            break;
                        case 'UL_EMPTY':
                            if( $a_ini_file[$a_tmp[0]][$a_tmp[1]] === 'PDEL-NULL' ) break;
                            $a_empty1 = explode('|' , $a_ini_file[$a_tmp[0]][$a_tmp[1]]);
                            for($i_ss2=0;$i_ss2<count($a_empty1);$i_ss2++){
                                $a_empty2 = explode(':' , $a_empty1[$i_ss2]);
                                if( count($a_empty2) !== 2 ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' format error');
                                switch($a_empty2[0]){
                                    case 'null':
                                        $a_empty2[0] = null;
                                        break;
                                    case 'null_str':
                                        $a_empty2[0] = '';
                                        break;
                                    case 'zero':
                                        $a_empty2[0] = 0;
                                        break;
                                    case 'false':
                                        $a_empty2[0] = false;
                                        break;
                                    case 'true':
                                        $a_empty2[0] = true;
                                        break;
                                    default:
                                        throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' format error');
                                }
                                switch($a_empty2[1]){
                                    case 'false':
                                        $a_empty2[1] = false;
                                        break;
                                    case 'true':
                                        $a_empty2[1] = true;
                                        break;
                                    default:
                                        throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' format error');
                                }
                                $this->_a_user_level_empty[$i_ss2] = $a_empty2;
                            }
                            break;
                    }
                    break;
                case 'C':
                    if( is_string($a_ini_file[$a_tmp[0]][$a_tmp[1]]) === false ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    if( strlen($a_ini_file[$a_tmp[0]][$a_tmp[1]]) !== 1 ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' def error');
                    switch($a_tmp[1]){
                        case 'UL_LIST_DEL':
                            if( ($a_ini_file[$a_tmp[0]][$a_tmp[1]] !== '') and ($this->_s_user_level_list === '') ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' must null');
                            if( ($a_ini_file[$a_tmp[0]][$a_tmp[1]] === '') and ($this->_s_user_level_list !== '') ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . $a_tmp[1] . ' must not null');
                            $this->_s_user_level_list_delimiter = $a_ini_file[$a_tmp[0]][$a_tmp[1]];
                            break;
                    }
                    break;
            }
        }
        if( ($this->_i_mask_type > 2) and ($this->_s_user_level_list !== 'PDEL-NULL') ){
            $i_ret = $this->_checkUserLevelListStr();
            if( $i_ret !== 0 ) throw new Exception($s_file_name . ' [' . $a_tmp[0] . '] ' . ' invalid UL_LIST(ret=' . $i_ret . ')');
        }
    }
    /**
     * 暗号化
     *
     * @param striing $s_target_string 暗号化前文字列
     * @param string  $s_recipe_name レシピ名
     * @param mixed   $m_opt1 オプション1(key_b=YYYYMMDDhhmmssまたはkey_c=XXXXXXXXXXXXXXXX)key_bは有効期限日時指定暗号化、key_cはセッションハイジャックチェック指定項目暗号化
     * @param mixed   $m_opt2 オプション2(指定時はkey=XXXXXXXXXXXXXXXX)
     * @return striing 暗号化後文字列
     */
    public function encrypt($s_target_string , $s_recipe_name , $m_opt1 = false , $m_opt2 = false)
    {
        if( $this->_b_debug_mode === true ) $this->_a_debug_inf = array();
        $a_ret = array();
        $a_ret['status'] = false;
        $a_ret['result'] = '';
        $a_ret['detail'] = '';
        $this->_s_result = '';
        try {
            if( strlen(mb_convert_encoding('a' , mb_internal_encoding())) !== 1 ) throw new Exception('invalid internal encoding');
            if( $this->_b_debug_mode === true ){
                $this->_f_debug_start_time = $this->_getCurrentTime4Debug(true);
                $this->_setDebugInf(__FUNCTION__ . '::' . __LINE__ . ' Recipe=' . $s_recipe_name . ' (Encrypt start at ' . date('Y/m/d H:i:s') . ')');
            }
            //空文字列は何もしない
            if( ($s_target_string === '') or ($s_target_string === null) ) return(array('status' => true , 'result' => ''));
            //復号モードオフ
            $this->_b_decrypt_mode = false;
            //パラメータチェック
            $b_status = $this->_checkParam($s_target_string , $s_recipe_name , $m_opt1 , $m_opt2);
            //暗号化
            if( $b_status === true ) $b_status = $this->_encryptMain($s_target_string);
            $a_ret['status'] = $b_status;
            if( $b_status === true ){
                $a_ret['result'] = $this->_s_result;
                if( $this->_s_auth_inf_b !== '' ) $a_ret['limit_time'] = $this->_GMT2local($this->_s_auth_inf_b);
            }
            else{
                $a_ret['detail'] = $this->_s_error_inf;
            }
            if( $this->_b_debug_mode === true ){
                $this->_setDebugInf('' , false , true);
                $a_ret['debug_inf'] = $this->_a_debug_inf;
            }
            $this->___error_proc($a_ret , $s_recipe_name);
            $this->_eraseVar();
            return $a_ret;
        } catch (Exception $exc) {
            $a_ret['status'] = false;
            $a_ret['result'] = '';
            $a_ret['detail'] = 'Z:### ' . __FUNCTION__ . '(' . __LINE__ . ')' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            if( $this->_b_debug_mode === true ) $a_ret['debug_inf'] = $this->_a_debug_inf;
            $this->___error_proc($a_ret , $s_recipe_name);
            $this->_eraseVar();
            return $a_ret;
        }
    }
    /**
     * 復号化
     *
     * @param striing $s_target_string 復号化前文字列
     * @param string $s_recipe_name レシピ名
     * @param mixed $m_user_level   ユーザー権限レベル
     * @param mixed $m_key_c　      セッションハイジャックチェックキー
     * @return striing 復号化後文字列
     */
    public function decrypt()
    {
        //Variant型データを定義できるDBも考慮し、以下のコメント行のような関数定義は行わない
        //public function decrypt($s_target_string , $s_recipe_name , $m_user_level = false , $m_key_c = false)
        if( $this->_b_debug_mode === true ) $this->_a_debug_inf = array();
        $a_ret = array();
        $a_ret['status'] = false;
        $a_ret['result'] = '';
        $a_ret['result_type'] = false;
        $a_ret['detail'] = '';
        $this->_s_result = '';
        $this->_i_mask_rule_index = -1;
        try {
            if( strlen(mb_convert_encoding('a' , mb_internal_encoding())) !== 1 ) throw new Exception('invalid internal encoding');
            $a_decrypt_args = func_get_args();
            switch(count($a_decrypt_args)){
                case 2:
                    $this->_m_param_user_level = false;
                    $this->_b_param_user_level_var = false;
                    break;
                case 3:
                case 4:
                    $this->_m_param_user_level = $a_decrypt_args[2];
                    $this->_b_param_user_level_var = true;
                    break;
                default:
                     throw new Exception('decrypt param count error');
            }
            if( $this->_b_debug_mode === true ){
                $this->_f_debug_start_time = $this->_getCurrentTime4Debug(true);
                $this->_setDebugInf(__FUNCTION__ . '::' . __LINE__ . ' Recipe=' . $a_decrypt_args[1] . ' (Decrypt start at ' . date('Y/m/d H:i:s') . ')');
            }
            //空文字列は何もしない
            if( ($a_decrypt_args[0] === '') or ($a_decrypt_args[0] === null) ) return(array('status' => true , 'result' => ''));
            //復号モードオン
            $this->_b_decrypt_mode = true;
            //パラメータチェック
            $b_status = $this->_checkParam($a_decrypt_args[0] , $a_decrypt_args[1] , false , false , $a_decrypt_args);
            //復号
            if( $b_status === true ){
                //マスク対象判定
                $b_mask = $this->_isMaskTarget($a_decrypt_args[1]);
                //復号
                if( ($b_mask === false) or ($b_mask === true and $this->_b_skip_decrypt === false) ) $b_status = $this->_decryptMain($a_decrypt_args[0]);
            }
            $a_ret['status'] = $b_status;
            if( $b_status === true ){
                if( $b_mask === false ){
                    $a_ret['result'] = $this->_s_result;
                    $a_ret['result_type'] = 0;
                }
                else{
                    $a_ret['result'] = $this->_a_mask_rule_file[$this->_i_mask_rule_index][1];
                    $a_ret['result_type'] = 1;
                }
            }
            else{
                if( ($this->_i_mask_rule_index !== -1) and (count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) > 2) ){
                    $a_ret['result'] = $this->_a_mask_rule_file[$this->_i_mask_rule_index][2];
                    $a_ret['result_type'] = 2;
                }
                else{
                    $a_ret['result'] = $this->_s_result;
                    $a_ret['result_type'] = 3;
                }
                $a_ret['detail'] = $this->_s_error_inf;
            }
            if( $this->_b_debug_mode === true ){
                $this->_setDebugInf('' , false , true);
                $a_ret['debug_inf'] = $this->_a_debug_inf;
            }
            $this->___error_proc($a_ret , $a_decrypt_args[1]);
            $this->_eraseVar();
            return $a_ret;
        } catch (Exception $exc) {
            $a_ret['status'] = false;
            $a_ret['result'] = '';
            $a_ret['result_type'] = 4;
            $a_ret['detail'] = 'Z:### ' . __FUNCTION__ . '(' . __LINE__ . ')' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            $this->___error_proc($a_ret , $a_decrypt_args[1]);
            $this->_eraseVar();
            return $a_ret;
        }
    }
    /**
     * パラメータチェック
     *
     * @param striing $s_target_string 変換対象文字列
     * @param string  $s_recipe_name   レシピ名
     * @param mixed   $m_encrypt_opt1  暗号化オプション1(key_b=YYYYMMDDHHMMまたはkey_c=XXXXXXXXXXXXXXXX)
     * @param mixed   $m_encrypt_opt2  暗号化オプション2(指定時はkey_c=XXXXXXXXXXXXXXXX)
     * @param mixed   $a_decrypt_args  復号関数引数
     * @return boolean
     */
    private function _checkParam(&$s_target_string , &$s_recipe_name , $m_encrypt_opt1 = false , $m_encrypt_opt2 = false , &$a_decrypt_args = false)
    {
        try {
            //PDEL-NULLが指定されたINI項目についてセッターで設定されていることをチェック
            if( ($this->_i_mask_type === 2) and ($this->_i_mask_user_level_min === null) ) throw new Exception('setter not used');
            if( ($this->_i_mask_type > 2) and ($this->_s_user_level_list === 'PDEL-NULL') ) throw new Exception('setter not used');
            //ユーザー指定認証キー情報
            $this->_s_auth_inf_b = '';
            $this->_s_user_key_c = '';
            //文字列チェック
            if( is_string($s_target_string) === false ) throw new Exception('target_string not string');
            if( is_string($s_recipe_name) === false ) throw new Exception('recipe_name not string');
            //処理対象データレングス
            $i_len = strlen($s_target_string);
            if( $this->_b_decrypt_mode === false ){
                //暗号化文字列バイト数チェック
                if( $i_len > self::ENCRYPT_MAX_SIZE ) throw new Exception('encrypt size error(' . $i_len . ')');
                //暗号化固有チェック
                //if( ($m_encrypt_opt1 === false) and ($m_encrypt_opt2 === false) ) return true;
                if( ($m_encrypt_opt1 === false) and ($m_encrypt_opt2 === false) ){
                    //なにもしない
                }
                else{
                    if( is_string($m_encrypt_opt1) === false ) throw new Exception('invalid encrypt opt1 param(not string)');
                    if( strpos($m_encrypt_opt1 , chr(0)) !== false ) throw new Exception('invalid encrypt opt1 param(null exists)');
                    if( strlen($m_encrypt_opt1) < 7 ) throw new Exception('invalid encrypt opt1 param(length)');
                    switch(substr($m_encrypt_opt1 , 0 , 6)){
                        case 'key_b=':
                            $this->_s_auth_inf_b = substr($m_encrypt_opt1 , 6);
                            $i_ret = $this->_checkDateTime($this->_s_auth_inf_b);
                            if( $i_ret !== 0 ) throw new Exception('invalid encrypt opt1 param(val=' . $this->_s_auth_inf_b . ' result=' . $i_ret . ')');
                            //local2GMT
                            $this->_s_auth_inf_b = $this->_local2GMT($this->_s_auth_inf_b);
                            break;
                        case 'key_c=':
                            $this->_s_user_key_c = substr($m_encrypt_opt1 , 6);
                            break;
                        default:
                             throw new Exception('invalid encrypt opt1 param(key)');
                    }
                    if( $m_encrypt_opt2 !== false ){
                        if( is_string($m_encrypt_opt2) === false ) throw new Exception('invalid encrypt opt1 param(not string)');
                        if( strpos($m_encrypt_opt2 , chr(0)) !== false ) throw new Exception('invalid encrypt opt2 param(null exists)');
                        if( strlen($m_encrypt_opt2) < 7 ) throw new Exception('invalid encrypt opt2 param(length)');
                        if( $this->_s_user_key_c !== '' ) throw new Exception('invalid encrypt opt2 param(dup)');
                        $this->_s_user_key_c = substr($m_encrypt_opt2 , 6);
                    }
                }
            }
            else{
                //復号化データチェック
                if( strlen($s_target_string) > self::DECRYPT_MAX_SIZE ) throw new Exception('decrypt size error');
                //使用文字チェック
                $s_ptn = '/^[^0-9a-zA-Z' . "\\" . self::SPECIAL_CHAR1 . "\\" . self::SPECIAL_CHAR2 . ']+$/';
                if( preg_match($s_ptn, $s_target_string) !== 0 ) throw new Exception('Invalid decrypt data');
                //復号パラメータ確定
                for($i_ss=2;$i_ss<count($a_decrypt_args);$i_ss++){
                    if( (is_string($a_decrypt_args[$i_ss]) === true) and (strlen($a_decrypt_args[$i_ss]) > 6) and (substr($a_decrypt_args[$i_ss] , 0 , 6) === 'key_c=') ){
                        if( $this->_s_user_key_c !== '' ) throw new Exception('invalid decrypt key_c param(dup)');
                        $this->_s_user_key_c = substr($a_decrypt_args[$i_ss] , 6);
                        if( strpos($this->_s_user_key_c , chr(0)) !== false ) throw new Exception('invalid decrypt key_b param(null exists)');
                        if( ($i_ss + 1) !== count($a_decrypt_args) ) throw new Exception('invalid decrypt param list');
                        if( $i_ss === 2 ){
                            $this->_m_param_user_level = false;
                            $this->_b_param_user_level_var = false;
                        }
                    }
                }
                //INIファイルとの整合性チェック
                if( ($this->_i_mask_type === 0) and ($this->_b_param_user_level_var === true) ) throw new Exception('invalid decrypt param list');
                //権限レベル指定時の範囲チェック
                if( ($this->_b_param_user_level_var === true) and ($this->_i_mask_type > 1) ){
                    $b_match = false;
                    if( $this->_i_mask_type === 2 ){
                        //NUM型の場合
                        if( is_int($this->_m_param_user_level) === true ){
                            //最小値～最大値の範囲内であること
                            if( ($this->_m_param_user_level < $this->_i_mask_user_level_min) or ($this->_m_param_user_level > $this->_i_mask_user_level_max) ){
                                throw new Exception('invalid decrypt param list');
                            }
                        }
                        else{
                            //Emptyチェック
                            for( $i_ss=0 ; $i_ss < count($this->_a_user_level_empty);$i_ss++){
                                if( $this->_m_param_user_level === $this->_a_user_level_empty[$i_ss][0] ){
                                    $b_match = true;
                                    $this->_m_param_user_level = $this->_a_user_level_empty[$i_ss][1];
                                    break;
                                }
                            }
                            if( $b_match === false ) throw new Exception('invalid decrypt param list');
                        }
                    }
                    else{
                        //NUM_STR・STR型の場合、権限リストに存在すること
                        $a_user_level = explode($this->_s_user_level_list_delimiter , $this->_s_user_level_list);
                        for( $i_ss=0 ; $i_ss < count($a_user_level);$i_ss++){
                            if( $this->_i_mask_type === 3 ){
                                //NUM_STR型の場合
                                if( (is_int($this->_m_param_user_level) === true) and ($this->_m_param_user_level === intval($a_user_level[$i_ss])) ){
                                    $b_match = true;
                                    $this->_m_param_user_level = $i_ss;
                                    break;
                                }
                            }
                            else{
                                //STR型の場合
                                if( $this->_m_param_user_level === $a_user_level[$i_ss] ){
                                    $b_match = true;
                                    $this->_m_param_user_level = $i_ss;
                                    break;
                                }
                            }
                        }
                        if( $b_match === false ){
                            //Emptyチェック
                            for( $i_ss=0 ; $i_ss < count($this->_a_user_level_empty);$i_ss++){
                                if( $this->_m_param_user_level === $this->_a_user_level_empty[$i_ss][0] ){
                                    $b_match = true;
                                    $this->_m_param_user_level = $this->_a_user_level_empty[$i_ss][1];
                                    break;
                                }
                            }
                            if( $b_match === false ) throw new Exception('invalid decrypt param list');
                        }
                    }
                }
            }
            //レシピ名チェック
            $i_ret = $this->_checkRecipeName($s_recipe_name);
            if( $i_ret !== 0 ) throw new Exception('RecipeName(' . $i_ret . ')');
            //固定モード設定
            $this->_b_fixed_mode = ($s_recipe_name[0] === self::RECIPE_PTN_F) ? true : false;
            //レシピファイルチェック
            $this->_s_target_recipe = '';
            if( $this->_checkRecipeFile($s_recipe_name) === false ) return false;
            if( $this->_a_recipe_inf[0][0] === 'ChangeAuthMode' ){
                if( $this->_checkRecipeFile($this->_s_target_recipe) === false ) return false;
            }
            else{
                $this->_s_target_recipe = $s_recipe_name;
            }
            //マップファイルチェック
            if( $this->_checkMapFile($this->_s_target_recipe) === false ) return false;
            //対象文字列がNULLを含んでいないことをチェック
            if( ($this->_b_bin_mode === false) and (strpos($s_target_string , chr(0)) !== false) ) throw new Exception('NULL exists');
            //ブラウザチェック
            if( $this->_i_auth_inf_c_mode !== 0 ){
                $i_ret = $this->_checkSessionHijackCheckKey();
                if( $i_ret !== 0 ){
                    $this->_s_error_inf = 'D:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . 'invalid browser(' . $i_ret . ')';
                    return false;
                }
            }
            else{
                $this->_s_auth_inf_c = '';
            }
            return true;
        } catch (Exception $exc) {
            $this->_s_error_inf = 'A:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            return false;
        }
    }
    /**
     * レシピ名チェック
     *
     * @param string $s_recipe_name レシピ名
     * @return int
     */
    private function _checkRecipeName($s_recipe_name)
    {
        //レシピ名で使用可能な文字は数字・アルファベット・記号_-[]
        $s_ptn = '/^[' . self::RECIPE_PTN_F . self::RECIPE_PTN_V . '][0-9a-zA-Z\_\-\[\]]+$/';
        if( preg_match($s_ptn, $s_recipe_name) !== 1 ) return 1;
        if( strlen($s_recipe_name) > self::RECIPE_MAX_LEN ) return 2;
        return 0;
    }
    /**
     * マップファイルチェック
     *
     * @param string $s_recipe_name レシピ名
     * @return boolean
     */
    private function _checkMapFile($s_recipe_name)
    {
        try {
            //マップファイル検索
            $s_file_name = $this->_s_mandatory_file_dir . self::MAP_FILE_PRE . $s_recipe_name . self::MAP_FILE_EXT;
            //ファイルステータスチェック
            $i_ret = $this->_checkFileStatus($s_file_name);
            if( $i_ret !== 0 ) throw new Exception('MAP file status error(' . $i_ret . ')');
            $this->_s_map_str = file_get_contents($s_file_name);
            if( $this->_s_map_str === false) throw new Exception('MAP file read error');
            //使用文字チェック
            $s_ptn = '/^[^0-9a-zA-Z' . "\\" . self::SPECIAL_CHAR1 . "\\" . self::SPECIAL_CHAR2 . ']+$/';
            if( preg_match($s_ptn, $this->_s_map_str) !== 0 ) throw new Exception('Invalid MAP(char)');
            //レングスチェック
            if( $this->_b_fixed_mode === true ){
                //固定暗号化の場合は144バイトかつ先頭64バイトとその後の64バイトが異なる内容であること
                if( strlen($this->_s_map_str) !== ((self::MAP_CHAR_NUM * 2) + self::FIXED_MAP_SS_MAX) ) throw new Exception('Invalid F-MAP(length)');
                $this->_s_extend_str = substr($this->_s_map_str , self::MAP_CHAR_NUM , self::MAP_CHAR_NUM);
                $this->_s_map_fixed_ss = substr($this->_s_map_str , (self::MAP_CHAR_NUM * 2));
                $this->_s_map_str = substr($this->_s_map_str , 0 , self::MAP_CHAR_NUM);
                if( strcmp($this->_s_extend_str , $this->_s_map_str) === 0 ) throw new Exception('Invalid F-MAP(map=extend)');
            }
            else{
                //可変暗号化の場合は64バイトであること
                if( strlen($this->_s_map_str) !== self::MAP_CHAR_NUM ) throw new Exception('Invalid V-MAP(length)');
            }
            //MAP内容チェック
            $a_char_cnt = array();
            for($i_ss1 = 0 ; $i_ss1 < 3 ; $i_ss1++){
                switch($i_ss1){
                    case 0:
                        //先頭64バイト
                        $s_chk_str = $this->_s_map_str;
                        break;
                    case 1:
                        //次の64バイト
                        $s_chk_str = $this->_s_extend_str;
                        break;
                    default:
                        //最後の16バイト
                        $s_chk_str = $this->_s_map_fixed_ss;
                }
                if( $i_ss1 < 2){
                    //64種チェック(64文字で重複がないこと)
                    for($i_ss2 = 0 ; $i_ss2 < self::MAP_CHAR_NUM ; $i_ss2++){
                        $a_char_cnt[$i_ss2] = 0;
                        for($i_ss3 = 0 ; $i_ss3 < self::MAP_CHAR_NUM ; $i_ss3++){
                            if( $this->_ASCII_STR[$i_ss2] === $this->_s_map_str[$i_ss3] ){
                                $a_char_cnt[$i_ss2]++;
                                //重複時は$a_char_cnt[$i_ss2]===0となる文字種別が存在するのでbreak
                                break;
                            }
                        }
                    }
                    for($i_ss2 = 0 ; $i_ss2 < self::MAP_CHAR_NUM ; $i_ss2++){
                        if( $a_char_cnt[$i_ss2] !== 1 ){
                            if( $i_ss1 === 1 ){
                                throw new Exception('Invalid MAP(extend char def)');
                            }
                            else{
                                throw new Exception('Invalid MAP(char def)');
                            }
                        }
                    }
                }
                else{
                    //4種チェック(0-3が均等に存在すること)
                    $s_ptn = '/^[^0-3]+$/';
                    if( preg_match('/^[^0-3]+$/', $s_chk_str) !== 0 ) throw new Exception('Invalid MAP(Fixed sub script)');
                    $a_char_cnt = array();
                    for($i_ss2 = 0 ; $i_ss2 < 4 ; $i_ss2++){
                        $a_char_cnt[$i_ss2] = 0;
                    }
                    for($i_ss2 = 0 ; $i_ss2 < self::FIXED_MAP_SS_MAX ; $i_ss2++){
                        $i_ss3 = intval($s_chk_str[$i_ss2]);
                        $a_char_cnt[$i_ss3]++;
                    }
                    for($i_ss2 = 0 ; $i_ss2 < 4 ; $i_ss2++){
                        if( $a_char_cnt[$i_ss2] !== 4 ) throw new Exception('Invalid MAP(Fixed sub script balance)');
                    }
                }
                if( $this->_b_fixed_mode === false ) break;
            }
            return true;
        } catch (Exception $exc) {
            $this->_s_error_inf = 'B:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            return false;
        }
    }
    /**
     * レシピファイルチェック
     *
     * @param string $s_recipe_name レシピ名
     * @return boolean
     */
    private function _checkRecipeFile($s_recipe_name)
    {
        try {
            $this->_b_bin_mode = false;
            $this->_i_auth_inf_c_mode = 0;
            $this->_s_hijack_check_ip = '';
            $this->_b_hijack_check_ua = false;
            $this->_b_hijack_check_lang = false;
            $b_apl_check = false;
            $this->_b_limit_by_encrypt_param = null;

            //レシピファイル検索
            if( $this->_readRecipeFile($s_recipe_name) === false ) return false;
            //レシピファイルチェック
            for($i_ss1 = 0 ; $i_ss1 < $this->_i_recipe_max ; $i_ss1++){
                switch($this->_a_recipe_inf[$i_ss1][0]){
                    case 'ChangeAuthMode':
                        //ChangeAuthModeで指定されたレシピにChangeAuthModeは存在してはならない
                        if( $this->_s_target_recipe !== '' ) throw new Exception('target recipe');
                        //ChangeAuthModeは1行だけの内容であること
                        if( $this->_i_recipe_max !== 1 ) throw new Exception('1 line rule');
                        if( $i_ss1 !== 0 ) throw new Exception('pos');
                        //ChangeAuthModeはパラメータが二つだけ定義されていること
                        if( count($this->_a_recipe_inf[$i_ss1]) !== 3 ) throw new Exception('param');
                        //ChangeAuthModeパラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , 0 , 7);
                        if( $i_ret !== 0 ) throw new Exception('num(' . $i_ret . ')');
                        if( $this->_i_default_auth_mode === intval($this->_a_recipe_inf[$i_ss1][1]) ) throw new Exception('new AuthMode must be not default');
                        $this->_i_default_auth_mode = intval($this->_a_recipe_inf[$i_ss1][1]);
                        //レシピ名チェック
                        $i_ret = $this->_checkRecipeName($this->_a_recipe_inf[$i_ss1][2]);
                        if( $i_ret !== 0 ) throw new Exception('RecipeName(' . $i_ret . ')');
                        if( $s_recipe_name[0] !== $this->_a_recipe_inf[$i_ss1][2][0] ) throw new Exception('RecipeType');
                        if( $s_recipe_name === $this->_a_recipe_inf[$i_ss1][2] ) throw new Exception('RecipeLoop');
                        //マップファイル検索
                        $s_file_name = $this->_s_mandatory_file_dir . self::MAP_FILE_PRE . $s_recipe_name . self::MAP_FILE_EXT;
                        //MAPファイルステータスチェック(存在しないことのチェック)
                        $i_ret = $this->_checkFileStatus($s_file_name);
                        if( $i_ret !== 1 ) throw new Exception('Map must not exists(' . $i_ret . ')');
                        $this->_s_target_recipe = $this->_a_recipe_inf[$i_ss1][2];
                        if( $this->_b_debug_mode === true ) $this->_setDebugInf(__FUNCTION__ . '::' . __LINE__ . ' ' . implode(':', $this->_a_recipe_inf[$i_ss1]));
                        break;
                    case 'CheckLimitTime':
                        //CheckLimitTimeは固定モードでは指定できない
                        if( $this->_b_fixed_mode === true ) throw new Exception('CheckLimitTime can not defined at fixed mode');
                        //CheckLimitTimeは先頭行にしか指定できない
                        if( $i_ss1 !== 0 ) throw new Exception('pos');
                        //CheckLimitTimeはパラメータが未定義または一つだけ定義されていること
                        switch(count($this->_a_recipe_inf[$i_ss1])){
                            case 1:
                                //レシピファイルのパラメータ未指定時はencrypt関数のkey_bで指定された引数を有効期限とする
                                $this->_b_limit_by_encrypt_param = true;
                                //過去日付指定の暗号化チェック
                                $s_current = $this->_getDateTime();
                                if( $s_current === '' ) throw new Exception('date cal error');
                                if( ($this->_b_decrypt_mode === false) and (strcmp($s_current , $this->_s_auth_inf_b) >= 0) ) throw new Exception('invalid key_b');
                                break;
                            case 2:
                                //レシピファイルのパラメータ指定時は現在時刻に指定秒数を加算した時刻を有効期限とする
                                $this->_b_limit_by_encrypt_param = false;
                                $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , 1 , self::LIMIT_MAX);
                                if( $i_ret !== 0 ) throw new Exception('T num(' . $i_ret . ')');
                                $this->_s_auth_inf_b = $this->_getDateTime(intval($this->_a_recipe_inf[$i_ss1][1]));
                                if( $this->_s_auth_inf_b === '' ) throw new Exception('date cal error');
                                break;
                            default:
                                throw new Exception('param');
                        }
                        break;
                    case 'CheckHijack':
                        //CheckHijackは固定モードでは指定できない
                        if( $this->_b_fixed_mode === true ) throw new Exception('CheckHijack can not defined at fixed mode');
                        //CheckHijackは先頭行または2行目にしか指定できない
                        switch($i_ss1){
                            case 0:
                                break;
                            case 1:
                                //CheckHijackが2行目以降の場合、上の行はChangeAuthModeまたはCheckLimitTimeでなければならない
                                if( $this->_a_recipe_inf[$i_ss1 - 1][0] !== 'CheckLimitTime' ) throw new Exception('pos');
                                break;
                            default:
                                throw new Exception('pos');
                        }
                        //CheckHijackはパラメータが一つまたは二つ定義されていること
                        if( (count($this->_a_recipe_inf[$i_ss1]) !== 2 ) and (count($this->_a_recipe_inf[$i_ss1]) !== 3) ) throw new Exception('param');
                        $a_chk = explode('+' , $this->_a_recipe_inf[$i_ss1][1]);
                        for($i_ss2 = 0 ; $i_ss2 < count($a_chk) ; $i_ss2++){
                            switch($a_chk[$i_ss2]){
                                case 'ZFIP':
                                case 'IP':
                                    if( $this->_s_hijack_check_ip !== '' ) throw new Exception('IP dup');
                                    $this->_s_hijack_check_ip = $a_chk[$i_ss2];
                                    break;
                                case 'UA':
                                    if( $this->_b_hijack_check_ua === true ) throw new Exception('UA dup');
                                    $this->_b_hijack_check_ua = true;
                                    break;
                                case 'LANG':
                                    if( $this->_b_hijack_check_lang === true ) throw new Exception('LANG dup');
                                    $this->_b_hijack_check_lang = true;
                                    break;
                                case 'APL':
                                    if( $this->_s_user_key_c === '' ) throw new Exception('apl original check inf not setted');
                                    if( $b_apl_check === true ) throw new Exception('APL dup');
                                    $b_apl_check = true;
                                    break;
                                default:
                                    throw new Exception('invalid hijack check option');
                            }
                        }
                        //ハイジャックチェック用文字列の認証方式
                        if( count($this->_a_recipe_inf[$i_ss1]) === 2 ){
                            //未指定の場合、デフォルト認証方式は2以上でなければならない
                            if( ($this->_i_default_auth_mode === 0) or ($this->_i_default_auth_mode === 1) ) throw new Exception('must direct hijack check AuthMode');
                            $this->_i_auth_inf_c_mode = $this->_i_default_auth_mode;
                        }
                        else{
                            //指定時は2以上でなければならない
                            $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 2 , 7);
                            if( $i_ret !== 0 ) throw new Exception('auth num(' . $i_ret . ')');
                            //指定時はデフォルト認証方式と異なる値でなければならない
                            if( $this->_i_default_auth_mode === intval($this->_a_recipe_inf[$i_ss1][2]) ) throw new Exception('hijack check AuthMode must be not default');
                            $this->_i_auth_inf_c_mode = intval($this->_a_recipe_inf[$i_ss1][2]);
                        }
                        if( $this->_b_debug_mode === true ) $this->_setDebugInf(__FUNCTION__ . '::' . __LINE__ . ' ' . implode(':', $this->_a_recipe_inf[$i_ss1]));
                        break;
                    case 'AllowBinString':
                        //AllowBinStringより上の行はCheckLimitTimeまたはCheckHijackでなければならない
                        switch($i_ss1){
                            //先頭行のケース
                            case 0:
                                break;
                            //2～3行目のケース
                            case 1:
                            case 2:
                                switch($this->_a_recipe_inf[$i_ss1 - 1][0]){
                                    case 'CheckLimitTime':
                                    case 'CheckHijack':
                                        break;
                                    default:
                                        throw new Exception('pos');
                                }
                                break;
                            //ChangeAuthMode・CheckOptionとExtendの間に変換系コマンド存在のケース
                            default:
                                throw new Exception('pos');
                        }

                        //AllowBinStringはパラメータが未定義であること
                        if( count($this->_a_recipe_inf[$i_ss1]) !== 1 ) throw new Exception('param');
                        $this->_b_bin_mode = true;
                        break;
                    case 'Extend':
                        //Extendより上の行はCheckLimitTimeまたはCheckHijackでなければならない
                        switch($i_ss1){
                            //先頭行のケース
                            case 0:
                                break;
                            //2～3行目のケース
                            case 1:
                            case 2:
                                switch($this->_a_recipe_inf[$i_ss1 - 1][0]){
                                    case 'CheckLimitTime':
                                    case 'CheckHijack':
                                        break;
                                    default:
                                        throw new Exception('pos');
                                }
                                break;
                            //ChangeAuthMode・CheckOptionとExtendの間に変換系コマンド存在のケース
                            default:
                                throw new Exception('pos');
                        }
                        //Extendはパラメータが一つだけ定義されていること
                        if( count($this->_a_recipe_inf[$i_ss1]) !== 2 ) throw new Exception('param');
                        //Extendパラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , self::EXTEND_MIN , self::EXTEND_MAX);
                        if( $i_ret !== 0 ) throw new Exception('num(' . $i_ret . ')');
                        break;
                    case 'Add':
                        //Addはパラメータが一つまたは二つ定義されていること
                        $i_cnt = count($this->_a_recipe_inf[$i_ss1]);
                        if( ($i_cnt !== 2) and ($i_cnt !== 3) ) throw new Exception('param');
                        //Add第1パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , 1 , self::ADD_MAX);
                        if( $i_ret !== 0 ) throw new Exception('1st num(' . $i_ret . ')');
                        //Add第2パラメータチェック
                        if( $i_cnt === 2 ){
                            $this->_a_recipe_inf[$i_ss1][2] = 0;
                            break;
                        }
                        if( $this->_a_recipe_inf[$i_ss1][2] === 'LAST' ) break;
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 0 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('2nd num(' . $i_ret . ')');
                        break;
                    case 'StrCycle':
                        //StrCycleはパラメータが二つ～四つ定義されていること
                        $i_cnt = count($this->_a_recipe_inf[$i_ss1]);
                        if( ($i_cnt !== 3) and ($i_cnt !== 4) and ($i_cnt !== 5) ) throw new Exception('param');
                        //StrCycle第1パラメータチェック
                        if( ($this->_a_recipe_inf[$i_ss1][1] !== 'L') and
                            ($this->_a_recipe_inf[$i_ss1][1] !== 'R') )
                            throw new Exception('direction)');
                        //StrCycle第2パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 1 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('2nd num(' . $i_ret . ')');
                        if( $i_cnt === 3 ){
                            $this->_a_recipe_inf[$i_ss1][3] = 0;
                            $this->_a_recipe_inf[$i_ss1][4] = 0;
                            break;
                        }
                        //StrCycle第3パラメータチェック
                        if( $this->_a_recipe_inf[$i_ss1][3] !== 'LAST' ){
                            $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][3] , 0 , (self::DECRYPT_MAX_SIZE - 1));
                            if( $i_ret !== 0 ) throw new Exception('3rd num(' . $i_ret . ')');
                        }
                        if( $i_cnt === 4 ){
                            $this->_a_recipe_inf[$i_ss1][4] = 0;
                            break;
                        }
                        //StrCycle第4パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][4] , 2 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('4th num(' . $i_ret . ')');
                        //第4パラメータ(循環範囲)指定時は第2パラメータ(循環文字数)と異なる内容であること
                        if( intval($this->_a_recipe_inf[$i_ss1][2]) === intval($this->_a_recipe_inf[$i_ss1][4]) ) throw new Exception('num relation error');
                        break;
                    case 'StrReverse':
                        //StrReverseはパラメータが未定義または一つまたは二つ定義されていること
                        $i_cnt = count($this->_a_recipe_inf[$i_ss1]);
                        if( ($i_cnt !== 1) and ($i_cnt !== 2) and ($i_cnt !== 3) ) throw new Exception('param');
                        if( $i_cnt === 1 ){
                            $this->_a_recipe_inf[$i_ss1][1] = 0;
                            $this->_a_recipe_inf[$i_ss1][2] = 0;
                            break;
                        }
                        //StrReverse第1パラメータチェック
                        if( $this->_a_recipe_inf[$i_ss1][1] !== 'LAST' ){
                            $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , 0 , (self::DECRYPT_MAX_SIZE - 1));
                            if( $i_ret !== 0 ) throw new Exception('1st num(' . $i_ret . ')');
                        }
                        if( $i_cnt === 2 ){
                            $this->_a_recipe_inf[$i_ss1][2] = 0;
                            break;
                        }
                        //StrReverse第2パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 2 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('2nd num(' . $i_ret . ')');
                        break;
                    case 'CharCycleByMap':
                    case 'CharCycleByAscii':
                        //CharCycleByMap・CharCycleByAsciiはパラメータが二つ～四つ定義されていること
                        $i_cnt = count($this->_a_recipe_inf[$i_ss1]);
                        if( ($i_cnt !== 3) and ($i_cnt !== 4) and ($i_cnt !== 5) ) throw new Exception('param');
                        //CharCycleByMap・CharCycleByAscii第1パラメータチェック
                        if( ($this->_a_recipe_inf[$i_ss1][1] !== 'L') and
                            ($this->_a_recipe_inf[$i_ss1][1] !== 'R') )
                            throw new Exception('direction)');
                        //CharCycleByMap・CharCycleByAscii第2パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 1 , 63);
                        if( $i_ret !== 0 ) throw new Exception('2nd num(' . $i_ret . ')');
                        if( $i_cnt === 3 ){
                            $this->_a_recipe_inf[$i_ss1][3] = 0;
                            $this->_a_recipe_inf[$i_ss1][4] = 0;
                            break;
                        }
                        //CharCycleByMap・CharCycleByAscii第3パラメータチェック
                        if( $this->_a_recipe_inf[$i_ss1][3] !== 'LAST' ){
                            $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][3] , 0 , (self::DECRYPT_MAX_SIZE - 1));
                            if( $i_ret !== 0 ) throw new Exception('3rd num(' . $i_ret . ')');
                        }
                        if( $i_cnt === 4 ){
                            $this->_a_recipe_inf[$i_ss1][4] = 0;
                            break;
                        }
                        //CharCycleByMap・CharCycleByAscii第4パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][4] , 1 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('4th num(' . $i_ret . ')');
                        break;
                    case 'CharReverseByMap':
                    case 'CharReverseByAscii':
                        //CharReverseByMap・CharReverseByAsciiはパラメータが未定義または一つまたは二つ定義されていること
                        $i_cnt = count($this->_a_recipe_inf[$i_ss1]);
                        if( ($i_cnt !== 1) and ($i_cnt !== 2) and ($i_cnt !== 3) ) throw new Exception('param');
                        if( $i_cnt === 1 ){
                            $this->_a_recipe_inf[$i_ss1][1] = 0;
                            $this->_a_recipe_inf[$i_ss1][2] = 0;
                            break;
                        }
                        //CharReverseByMap・CharReverseByAscii第1パラメータチェック
                        if( $this->_a_recipe_inf[$i_ss1][1] !== 'LAST' ){
                            $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][1] , 0 , (self::DECRYPT_MAX_SIZE - 1));
                            if( $i_ret !== 0 ) throw new Exception('1st num(' . $i_ret . ')');
                        }
                        if( $i_cnt === 2 ){
                            $this->_a_recipe_inf[$i_ss1][2] = 0;
                            break;
                        }
                        //CharReverseByMap・CharReverseByAscii第2パラメータチェック
                        $i_ret = $this->_checkDefFileNum($this->_a_recipe_inf[$i_ss1][2] , 1 , (self::DECRYPT_MAX_SIZE - 1));
                        if( $i_ret !== 0 ) throw new Exception('2nd num(' . $i_ret . ')');
                        break;
                    default:
                        $s_msg = 'Invalid command(' . $this->_a_recipe_inf[$i_ss1][0] . ')';
                        $this->_a_recipe_inf[$i_ss1][0] = '####';
                        throw new Exception($s_msg);
                }
            }
            if( $this->_a_recipe_inf[0][0] !== 'ChangeAuthMode' ){
                $this->_i_cnv_cmd_start = -1;
                //レシピファイルチェックに変換系コマンドが1件以上定義されていることをチェック
                for($i_ss1 = 0 ; $i_ss1 < $this->_i_recipe_max ; $i_ss1++){
                    switch($this->_a_recipe_inf[$i_ss1][0]){
                        case 'CheckLimitTime':
                        case 'CheckHijack':
                        case 'AllowBinString':
                            break;
                        default:
                            $this->_i_cnv_cmd_start = $i_ss1;
                    }
                    if( $this->_i_cnv_cmd_start !== -1 ) break;
                }
                if( $this->_i_cnv_cmd_start === -1 ){
                    $i_ss1 = 0;
                    throw new Exception('convert cmd not defined');
                }
            }
            if( ($this->_b_limit_by_encrypt_param === null) and ($this->_s_auth_inf_b !== '') ) throw new Exception('key_b can not directed');
            if( ($b_apl_check === false) and ($this->_s_user_key_c !== '') )throw new Exception('key_c can not directed');
            return true;
        } catch (Exception $exc) {
            $s_msg = 'Invalid Recipe(valid line=' . ($i_ss1 + 1) . ')' . $this->_a_recipe_inf[$i_ss1][0] . ':';
            $this->_s_error_inf = 'C:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $s_msg . $exc->getMessage();
            return false;
        }
    }
    /**
     * ファイルステータスチェック
     *
     * @param string  $s_file_name チェック対象フルパスファイル名
     * @param boolean $b_mask_file マスクルールファイル区分
     * @return int   0:正常 1:存在しない 2:読み込み権限なし 3:権限情報取得エラー 4:権限不正(400・000以外) 5:シンボリックリンク(ディレクトリ) 6:シンボリックリンク(ファイル) 7:stat関数エラー 8:ハードリンク数情報なし 9:ハードリンク数2以上
     */
    private function _checkFileStatus($s_file_name , $b_mask_file = false)
    {
        //ファイル存在チェック
        if( (file_exists($s_file_name) === false) or (is_file($s_file_name) === false) ) return 1;
        //読み込み権限が存在することのチェック
        if( is_readable($s_file_name) === false ) return 2;
        //ファイルステータスチェック
        $i_check_val = $b_mask_file ? $this->_i_mask_rule_file_check_type : $this->_i_mandatory_file_check_type;
        //権限が000または400であることのチェック
        if( $i_check_val >= 8 ){
            $i_fp = fileperms($s_file_name);
            if( $i_fp === false ) return 3;
            $s_fp = substr(sprintf('%03o' , $i_fp) , -3);
            switch($s_fp){
                case '000':
                case '400':
                    $i_check_val -= 8;
                    break;
                default:
                    return 4;
            }
        }
        //ファイル配置ディレクトリがシンボリックリンクでないことのチェック
        if( $i_check_val >= 4 ){
            $s_dir = $b_mask_file ? $this->_s_mask_rule_file_dir : $this->_s_mandatory_file_dir;
            $s_dir = $s_dir . '.';
            $s_real_path = realpath($s_dir);
            $s_real_path = str_replace(DIRECTORY_SEPARATOR , '/' , $s_real_path);
            if( substr($s_real_path , -1) !== '/' ) $s_real_path = $s_real_path . '/';
            if( stripos(PHP_OS , 'WIN') === false ){
                //Windows以外は大文字小文字を区別する
                if( $s_real_path !== $s_dir ) return 5;
            }
            else{
                //Windowsは大文字小文字を区別しない
                if( strcasecmp($s_real_path , $s_dir) !== 0 ) return 5;
            }
            $i_check_val -= 4;
        }
        //ファイル本体がシンボリックリンクでないことのチェック
        if( $i_check_val >= 2 ){
            if( is_link($s_file_name) === true ) return 6;
            $i_check_val -= 2;
        }
        //ハードリンク数が1であることのチェック
        if( $i_check_val === 1 ){
            $a_stat = stat($s_file_name);
            if( $a_stat === false ) return 7;
            if( array_key_exists('nlink' , $a_stat) === false ) return 8;
            if( $a_stat['nlink'] !== 1 ) return 9;
        }
        return 0;
    }
    /**
     * 定義ファイル用数値チェック
     *
     * @param string $s_num チェック対象数値
     * @param int    $i_min 最小値
     * @param int    $i_max 最大値
     * @return int   0:正常 1:数字以外 2:最小値未満 3:最大値超過
     */
    private function _checkDefFileNum($s_num , $i_min , $i_max)
    {
        //数字であること
        if(ctype_digit($s_num) === false ) return 1;
        $i_def_num = intval($s_num);
        //最小値
        if( $i_def_num < $i_min ) return 2;
        //最大値
        if( $i_def_num > $i_max ) return 3;
        return 0;
    }
    /**
     * レシピファイルREAD
     *
     * @param string $s_recipe_name レシピ名
     * @return boolean
     */
    private function _readRecipeFile($s_recipe_name)
    {
        try {
            //レシピファイルの有効行をデリミタ:でexplode後、配列にセット
            //レシピ使用可能文字種別のチェック、および件数のチェックはここで行う
            $s_file_name = $this->_s_mandatory_file_dir . self::RECIPE_FILE_PRE . $s_recipe_name . self::RECIPE_FILE_EXT;
            //ファイルステータスチェック
            $i_ret = $this->_checkFileStatus($s_file_name);
            if( $i_ret !== 0 ) throw new Exception('Recipe file status error(' . $i_ret . ')');
            $r_fp = fopen($s_file_name , 'r');
            if( $r_fp === false ) throw new Exception('Recipe file open error');
            $this->_i_recipe_max = 0;
            $this->_a_recipe_inf = array();
            //レシピ使用可能文字種別
            $s_ptn = '/^[^0-9a-zA-Z\:\+]+$/';
            while(true){
                $s_line = fgets($r_fp);
                if( $s_line !== false ){
                    $s_line = trim($s_line);
                    if( $s_line !== '' and $s_line[0] !== '#' ){
                        //使用文字チェック(英数字と:と+以外の文字種別はエラーとする)
                        if( preg_match($s_ptn , $s_line) !== 0 ){
                            fclose($r_fp);
                            throw new Exception('Invalid Recipe(char valid line=' . ($this->_i_recipe_max + 1) . ')');
                        }
                        //最大件数チェック
                        if( $this->_i_recipe_max === self::RECIPE_CMD_MAXF ){
                            fclose($r_fp);
                            throw new Exception('Invalid Recipe(too many cmd-lines)');
                        }
                        //配列セット
                        $this->_a_recipe_inf[$this->_i_recipe_max] = explode(':' , $s_line);
                        $this->_i_recipe_max = $this->_i_recipe_max + 1;
                    }
                }
                if( feof($r_fp) === true ) break;
            }
            fclose($r_fp);
            if( $this->_i_recipe_max === 0 ) throw new Exception('Recipe file not defined(recipe_type=' . $s_recipe_name . ')');
            //読み込みバッファを念のためスペースで置き換える
            $s_line = str_repeat(' ' , strlen($s_line));
            return true;
        } catch (Exception $exc) {
            $this->_s_error_inf = 'C:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            return false;
        }
    }
    /**
     * 暗号化メイン
     *
     * @param string $s_target_string 対象文字列
     * @return boolean
     */
    private function _encryptMain($s_target_string)
    {
        try {
            if( $this->_b_debug_mode === true ) $this->_setDebugInf(__FUNCTION__ . '::' . __LINE__ . ' Encrypt_target ' . $s_target_string);
            //暗号化前処理
            $s_converted = $this->_cmdBefore($s_target_string);
            for($this->_i_current_recipe = $this->_i_cnv_cmd_start ; $this->_i_current_recipe < $this->_i_recipe_max ; $this->_i_current_recipe++){
                switch($this->_a_recipe_inf[$this->_i_current_recipe][0]){
                    case 'Extend':
                        $s_converted = $this->_cmdExtend($s_converted);
                        break;
                    case 'Add':
                        $s_converted = $this->_cmdAdd($s_converted);
                        break;
                    case 'StrCycle':
                        $s_converted = $this->_cmdStrCycle($s_converted);
                        break;
                    case 'StrReverse':
                        $s_converted = $this->_cmdStrReverse($s_converted);
                        break;
                    case 'CharCycleByMap':
                        $s_converted = $this->_cmdCharCycle($s_converted , true);
                        break;
                    case 'CharCycleByAscii':
                        $s_converted = $this->_cmdCharCycle($s_converted , false);
                        break;
                    case 'CharReverseByMap':
                        $s_converted = $this->_cmdCharReverse($s_converted , true);
                        break;
                    default:
                        $s_converted = $this->_cmdCharReverse($s_converted , false);
                        break;
                }
                if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ , true);
            }
            $this->_s_result = $s_converted;
            return true;
        } catch (Exception $exc) {
            $this->_s_error_inf = 'E:' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            return false;
        }
    }
    /**
     * コマンド実行前処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdBefore($s_target_string)
    {
        //16進数化
        $s_converted = bin2hex($s_target_string);
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ . ' hex_data');
        //認証情報A(ハッシュ値)
        switch($this->_i_default_auth_mode){
            case 0:
            case 1:
                $s_auth_inf_a = '';
                break;
            case 2:
            case 5:
                $s_auth_inf_a = sprintf('%08x' , crc32($s_target_string));
                break;
            case 3:
            case 6:
                $s_auth_inf_a = md5($s_target_string);
                break;
            case 4:
            case 7:
                $s_auth_inf_a = sha1($s_target_string);
                break;
        }
        //認証情報A(レングス)
        if( ($this->_i_default_auth_mode === 1 ) or ( $this->_i_default_auth_mode >= 5) ) $s_auth_inf_a = sprintf('%04x' , strlen($s_target_string)) . $s_auth_inf_a;
        //認証情報B・Cはレシピチェック時に設定済み
        //認証情報付与
        $s_converted = $s_auth_inf_a . $this->_s_auth_inf_b . $this->_s_auth_inf_c . $s_converted;
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ . ' +auth_inf');
        if( $this->_a_recipe_inf[$this->_i_cnv_cmd_start][0] !== 'Extend' ){
            //Extend以外はここでマップ文字列に変換する
            $s_converted = $this->_hex2Map($s_converted);
        }
        return $s_converted;
    }
    /**
     * 16進数マップキャラ変換
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _hex2Map($s_target_string)
    {
        $s_converted = '';
        $s_quarter = '';
        $i_len = strlen($s_target_string);

        if( $this->_b_fixed_mode === false ){
            //可変暗号化の場合
            //0123からなる変換対象文字列と同じ長さのランダムな文字列を生成する
            $i_div = floor($i_len / 4);
            $i_rem = $i_len % 4;
            if( $i_div !== 0 ) $s_quarter = str_repeat('3210', $i_div);
            if( $i_rem !== 0 ) $s_quarter .= substr(str_shuffle('3210') , 0 , $i_rem);
            $s_quarter = str_shuffle($s_quarter);
        }
        else{
            //固定暗号化の場合
            //0123からなる変換対象文字列と同じ長さの固定文字列を生成する
            $i_div = floor($i_len / self::FIXED_MAP_SS_MAX);
            $i_rem = $i_len % self::FIXED_MAP_SS_MAX;
            if( $i_div !== 0 ) $s_quarter = str_repeat($this->_s_map_fixed_ss , $i_div);
            if( $i_rem !== 0 ) $s_quarter .= strrev(substr($this->_s_map_fixed_ss , (0 - $i_rem)));
        }
        for($i_ss=0 ; $i_ss < $i_len ; $i_ss++){
            $i_pos = (4 * (hexdec($s_target_string[$i_ss]))) + intval($s_quarter[$i_ss]);
            $s_converted .= $this->_s_map_str[$i_pos];
        }
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__);
        return $s_converted;
    }
    /**
     * マップキャラ16進数変換
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _map2Hex($s_target_string)
    {
        $s_converted = '';
        $i_len = strlen($s_target_string);
        for($i_ss=0 ; $i_ss < $i_len ; $i_ss++){
            $s_c = $s_target_string[$i_ss];
            $i_pos = strpos($this->_s_map_str , $s_c);
            if( $i_pos === false ) throw new Exception('### BUG ###');
            $i_pos = floor($i_pos / 4);
            $s_converted .= dechex($i_pos);
        }
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__);
        return $s_converted;
    }
    /**
     * 復号化メイン
     *
     * @param string $s_target_string 対象文字列
     * @return boolean
     */
    private function _decryptMain($s_target_string)
    {
        $this->_s_decrypt_err_category = 'I';
        try {
            if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_target_string . ' ' . __FUNCTION__ . '::' . __LINE__ . ' Decrypt_target' );
            //復号化
            $s_converted = $s_target_string;
            for($this->_i_current_recipe = $this->_i_recipe_max - 1 ; $this->_i_current_recipe >= $this->_i_cnv_cmd_start ; $this->_i_current_recipe--){
                switch($this->_a_recipe_inf[$this->_i_current_recipe][0]){
                    case 'Extend':
                        $s_converted = $this->_cmdExtend($s_converted);
                        break;
                    case 'Add':
                        $s_converted = $this->_cmdAdd($s_converted);
                        break;
                    case 'StrCycle':
                        $s_converted = $this->_cmdStrCycle($s_converted);
                        break;
                    case 'StrReverse':
                        $s_converted = $this->_cmdStrReverse($s_converted);
                        break;
                    case 'CharCycleByMap':
                        $s_converted = $this->_cmdCharCycle($s_converted , true);
                        break;
                    case 'CharCycleByAscii':
                        $s_converted = $this->_cmdCharCycle($s_converted , false);
                        break;
                    case 'CharReverseByMap':
                        $s_converted = $this->_cmdCharReverse($s_converted , true);
                        break;
                    default:
                        $s_converted = $this->_cmdCharReverse($s_converted , false);
                        break;
                }
                if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ , true);
            }
            $s_converted = $this->_cmdAfter($s_converted);
            $this->_s_result = $s_converted;
            return true;
        } catch (Exception $exc) {
            $this->_s_error_inf = $this->_s_decrypt_err_category . ':' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            return false;
        }
    }
    /**
     * コマンド実行後処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdAfter($s_target_string)
    {
        $s_converted = $s_target_string;
        //Extend以外はマップ文字列を16進数文字列に変換
        if( $this->_a_recipe_inf[$this->_i_cnv_cmd_start][0] !== 'Extend' ) $s_converted = $this->_map2Hex($s_converted);
        $i_len = strlen($s_converted);
        //奇数バイトはありえない
        if( ($i_len % 2) === 1 ) throw new Exception('decrypt error(invalid data length)');
        //認証情報総レングス
        $i_auth_inf_total_len = $this->_getAuthInfTotalLen();
        //認証情報以降にデータが存在することをチェック
        if( $i_len <= $i_auth_inf_total_len ) throw new Exception('decrypt error(valid data not exists)');
        //バイナリ文字列復号時以外は認証情報以降にNullが存在しないことをチェック
        if( $this->_b_bin_mode === false ){
            $i_pos = $i_auth_inf_total_len;
            while($i_pos < $i_len){
                if( substr($s_converted , $i_pos , 2) === '00' ) throw new Exception('decrypt error(null exists)');
                $i_pos = $i_pos + 2;
            }
        }
        //認証モードに応じて認証情報A取得
        switch($this->_i_default_auth_mode){
            case 0:
                $s_auth_inf_a_len_decrypted = '';
                $s_auth_inf_a_hash_decrypted = '';
            case 1:
                $s_auth_inf_a_hash_decrypted = '';
                $s_auth_inf_a_len_decrypted = substr($s_converted , 0 , 4);
                break;
            case 2:
            case 3:
            case 4:
                $s_auth_inf_a_len_decrypted = '';
                $s_auth_inf_a_hash_decrypted = substr($s_converted , 0 , $this->_a_auth_mode_len[$this->_i_default_auth_mode]);
                break;
            case 5:
            case 6:
            case 7:
                $s_auth_inf_a_len_decrypted = substr($s_converted , 0 , 4);
                $s_auth_inf_a_hash_decrypted = substr($s_converted , 4 , ($this->_a_auth_mode_len[$this->_i_default_auth_mode] - 4));
                break;
        }
        //認証情報B取得
        $s_auth_inf_b_decrypted = ($this->_b_limit_by_encrypt_param !== null) ? substr($s_converted , $this->_a_auth_mode_len[$this->_i_default_auth_mode] , 14) : '';
        //認証情報C取得
        if( $this->_i_auth_inf_c_mode !== 0 ){
            if( $this->_b_limit_by_encrypt_param === null ){
                $s_auth_inf_c_decrypted = substr($s_converted , $this->_a_auth_mode_len[$this->_i_default_auth_mode] , $this->_a_auth_mode_len[$this->_i_auth_inf_c_mode]);
            }
            else{
                $s_auth_inf_c_decrypted = substr($s_converted , ($this->_a_auth_mode_len[$this->_i_default_auth_mode] + 14) , $this->_a_auth_mode_len[$this->_i_auth_inf_c_mode]);
            }
        }
        else{
            $s_auth_inf_c_decrypted = '';
        }
        //復号データ取得
        $s_converted = substr($s_converted , $i_auth_inf_total_len);
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ . ' -auth_inf');
        //復元
        $s_converted = pack('H*' , $s_converted);
        //認証情報Aレングスチェック
        if( $s_auth_inf_a_len_decrypted !== '' ){
            $i_auth_inf_a_length_current = strlen($s_converted);
            if( hexdec($s_auth_inf_a_len_decrypted) !==  $i_auth_inf_a_length_current ) throw new Exception('decrypt error(length inf check error)');
        }
        //復号データハッシュ値取得
        switch($this->_i_default_auth_mode){
            case 2:
            case 5:
                $s_auth_inf_a_hash_current = sprintf('%08x' , crc32($s_converted));
                break;
            case 3:
            case 6:
                $s_auth_inf_a_hash_current = md5($s_converted);
                break;
            case 4:
            case 7:
                $s_auth_inf_a_hash_current = sha1($s_converted);
                break;
            default:
                $s_auth_inf_a_hash_current = '';
        }
        //認証情報Aハッシュ値チェック
        if( $s_auth_inf_a_hash_decrypted !== '' ){
            //認証情報はPDELがPHP以外で実装されることも考慮して大文字小文字を区別せずに行う
            if( strcasecmp($s_auth_inf_a_hash_decrypted , $s_auth_inf_a_hash_current) !== 0 ) throw new Exception('decrypt error(auth inf check error)');
        }
        //セッションハイジャックチェック(重大なエラーのため、有効期限よりも先にチェックする)
        if( $s_auth_inf_c_decrypted !== '' ){
            if( strcmp($s_auth_inf_c_decrypted , $this->_s_auth_inf_c) !== 0 ){
                $this->_s_decrypt_err_category = 'F';
                throw new Exception('decrypt error(session hijack check error)');
            }
        }
        //有効期限チェック
        if( $s_auth_inf_b_decrypted !== '' ){
            $i_ret = $this->_checkDateTime($s_auth_inf_b_decrypted);
            if( $i_ret !== 0 ){
                //認証情報AがOKにもかかわらず認証情報Bが日時でないエラーは極めて稀有なケースなため、エラーカテゴリーを分ける
                $this->_s_decrypt_err_category = 'G';
                throw new Exception('The Reckoning. The Tragedy and Lessons of PDEL. PDEL is hacked by the Best and the Brightest. (' . $s_auth_inf_b_decrypted . ')');
            }
            $s_now = $this->_getDateTime();
            if( $s_now === '' ){
                $this->_s_decrypt_err_category = 'Z';
                throw new Exception('date cal error');
            }
            if( strcmp($s_now , $s_auth_inf_b_decrypted) > 0 ){
                $this->_s_decrypt_err_category = 'H';
                throw new Exception('decrypt error(time check error limit=' . $this->_GMT2local($s_auth_inf_b_decrypted) . ')');
            }
        }
        if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__);
        return $s_converted;
    }
    /**
     * Extend処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdExtend($s_target_string)
    {
        //ExtendはNull(00)を文字列末尾に付与して行う
        //但し拡張前のサイズがExtendと同じであれば何もしない
        $s_converted = $s_target_string;
        if( $this->_b_decrypt_mode === false){
            //拡張サイズ
            $i_len = intval($this->_a_recipe_inf[$this->_i_current_recipe][1]) - strlen($s_converted);
            if( $i_len < 0 ){
                throw new Exception('Extend size error');
            }
            elseif( $i_len === 1 ){
                $s_converted .= '0';
            }
            elseif( $i_len > 1 ){
                $s_converted .= '00';
            }
            if( $this->_b_debug_mode === true ) $this->_setDebugInf($s_converted . ' ' . __FUNCTION__ . '::' . __LINE__ . ' +delimiter');
            //ExtendはNull付与後にマップ文字列変換
            $s_converted = $this->_hex2Map($s_converted);
            $i_len = $i_len - 2;
            if( $i_len > 0 ){
                //Extend用文字列取得
                if( $this->_b_fixed_mode  === false ){
                    //ランダム文字列取得
                    $s_extend_str = $this->_genRandomString($i_len);
                }
                else{
                    //固定文字列取得
                    $s_extend_str = $this->_genFixedString($i_len);
                }
                $s_converted .= $s_extend_str;
            }
        }
        else{
            //拡張後サイズ
            $i_len = intval($this->_a_recipe_inf[$this->_i_current_recipe][1]);
            $i_before_len = strlen($s_converted);
            if( $i_len !== $i_before_len )  throw new Exception('Extend decrypt error(data size)');
            //ExtendはNullサーチ前に16進数変換
            $s_converted = $this->_map2Hex($s_converted);
            //検索開始位置決定(認証情報+1文字分HEXより後方のNullをサーチ)
            $i_pos = $this->_getAuthInfTotalLen();
            $i_pos += 2;
            $b_trim = false;
            while(true){
                if( $i_pos >= $i_len) break;
                if( substr($s_converted , $i_pos , 2) === '00' ){
                    if( ($this->_b_fixed_mode  === true) and (($i_pos +2) > $i_len) ){
                        //固定変換の場合は復元した拡張文字列が付与されるものと同じであることをチェック
                        $s_decrypted_extend_str = substr($s_converted , ($i_pos + 2));
                        $s_extend_str = $this->_genFixedString($i_len);
                        if( strcmp($s_decrypted_extend_str , $s_extend_str) !== 0 ) throw new Exception('Extend decrypt error(fixed extend data)');
                    }
                    //Null直前までの文字列取得
                    $s_converted = substr($s_converted , 0 , $i_pos);
                    $b_trim = true;
                    break;
                }
                $i_pos += 2;
            }
            //奇数バイトの拡張かつNull未存在のケース
            if( ($b_trim === false) and (($i_before_len % 2) === 1) ){
                //奇数バイトの拡張かつNull未存在は拡張前サイズを1Byteだけ0を付与しているので、そのチェックを行う
                if( substr($s_converted , -1) !== '0' ) throw new Exception('Extend decrypt error(1 byte extend)');
                $s_converted = substr($s_converted , 0 , ($i_before_len - 1));
            }
        }
        return $s_converted;
    }
    /**
     * Add処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdAdd($s_target_string)
    {
        $s_converted = $s_target_string;
        if( $this->_b_decrypt_mode === false){
            //暗号化
            if( $this->_b_fixed_mode  === false ){
                //ランダム文字列取得
                $s_add_str = $this->_genRandomString(intval($this->_a_recipe_inf[$this->_i_current_recipe][1]));
            }
            else{
                //固定文字列取得
                $s_add_str = $this->_genFixedString(intval($this->_a_recipe_inf[$this->_i_current_recipe][1]));
            }
            //POS情報に応じて付与
            if( $this->_a_recipe_inf[$this->_i_current_recipe][2] === 'LAST' ){
                $s_converted .= $s_add_str;
            }
            else{
                $i_before_len = strlen($s_converted);
                $i_add_pos = intval($this->_a_recipe_inf[$this->_i_current_recipe][2]);
                $i_add_pos = ($i_add_pos % $i_before_len);
                if( $i_add_pos === 0 ){
                    $s_converted = $s_add_str . $s_converted;
                }
                else{
                    $s_converted = substr($s_converted , 0 , $i_add_pos) . $s_add_str . substr($s_converted , $i_add_pos);
                }
            }
        }
        else{
            //復号化
            $i_trim_len = intval($this->_a_recipe_inf[$this->_i_current_recipe][1]);
            $i_before_len = strlen($s_converted);
            $i_after_len = $i_before_len - $i_trim_len;
            if( $i_after_len <= 0 ) throw new Exception('Add decrypt error');
            //POS情報に応じて削除
            if( $this->_a_recipe_inf[$this->_i_current_recipe][2] === 'LAST' ){
                if( $this->_b_fixed_mode  === true ){
                    $s_decrypted_add_str = substr($s_converted , $i_after_len);
                }
                $s_converted = substr($s_converted , 0 , $i_after_len);
            }
            else{
                $i_trim_pos = intval($this->_a_recipe_inf[$this->_i_current_recipe][2]);
                $i_trim_pos = ($i_trim_pos % $i_after_len);
                if( $this->_b_fixed_mode  === true ){
                    $s_decrypted_add_str = substr($s_converted , $i_trim_pos , $i_trim_len);
                }
                if( $i_trim_pos === 0 ){
                    $s_converted = substr($s_converted , $i_trim_len);
                }
                else{
                    $s_converted = substr($s_converted , 0 , $i_trim_pos) . substr($s_converted , ($i_trim_pos + $i_trim_len));
                }
            }
            if( $this->_b_fixed_mode  === true ){
                //固定文字列取得
                $s_add_str = $this->_genFixedString(intval($this->_a_recipe_inf[$this->_i_current_recipe][1]));
                if( strcmp($s_decrypted_add_str , $s_add_str) !== 0 ) throw new Exception('Add decrypt error(fixed add data)');
            }
        }
        return $s_converted;
    }
    /**
     * StrCycle処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdStrCycle($s_target_string)
    {
        //対象領域確定
        $a_s_area = $this->_divString($s_target_string);
        $i_num = (intval($this->_a_recipe_inf[$this->_i_current_recipe][2]) % strlen($a_s_area[1]));
        if( (strlen($a_s_area[1]) > 1) and ($i_num > 0) ){
            //対象領域が2桁以上の場合
            if( (($this->_b_decrypt_mode === false) and ($this->_a_recipe_inf[$this->_i_current_recipe][1] === 'L')) or
                (($this->_b_decrypt_mode === true)  and ($this->_a_recipe_inf[$this->_i_current_recipe][1] === 'R')) ){
               $a_s_area[1] = substr($a_s_area[1] , $i_num) . substr($a_s_area[1] , 0 , $i_num);
            }
            else{
               $a_s_area[1] = substr($a_s_area[1] , (0 - $i_num)) . substr($a_s_area[1] , 0 , strlen($a_s_area[1]) - $i_num);
            }
        }
        $s_converted = $a_s_area[0] . $a_s_area[1] . $a_s_area[2];
        return $s_converted;
    }
    /**
     * 文字列分割処理
     *
     * @param string $s_target_string 対象文字列
     * @return array
     */
    private function _divString($s_target_string)
    {
        /**
         * ExtendとAdd以外の命令で、処理対象文字列について
         * 変換対象領域の左側
         * 変換対象領域
         * 変換対象領域の右側
         * に分割する
         */
        $a_s_area = array();
        $a_s_area[0] = '';
        $a_s_area[1] = '';
        $a_s_area[2] = '';
        $i_target_len = strlen($s_target_string);
        switch($this->_a_recipe_inf[$this->_i_current_recipe][0]){
            case 'StrCycle':
            case 'CharCycleByMap':
            case 'CharCycleByAscii':
                $i_start_pos = $this->_a_recipe_inf[$this->_i_current_recipe][3];
                $i_range_len = intval($this->_a_recipe_inf[$this->_i_current_recipe][4]);
                break;
            default:
                $i_start_pos = $this->_a_recipe_inf[$this->_i_current_recipe][1];
                $i_range_len = intval($this->_a_recipe_inf[$this->_i_current_recipe][2]);
                break;
        }
        if( $i_start_pos === 'LAST' ){
            if( ($i_range_len === 0) or ($i_range_len >= $i_target_len) ){
                $a_s_area[1] = $s_target_string;
            }
            else{
                //ABCDEFGH
                //LAST:3
                $a_s_area[0] = substr($s_target_string , 0 , ($i_target_len - $i_range_len));
                $a_s_area[1] = substr($s_target_string , (0 - $i_range_len));
            }
        }
        else{
            $i_start_pos = (intval($i_start_pos) % $i_target_len);
            if( $i_start_pos === 0 ){
                if( ($i_range_len === 0) or ($i_range_len >= $i_target_len) ){
                    $a_s_area[1] = $s_target_string;
                }
                else{
                    //ABCDEFGH
                    //0:3
                    $a_s_area[1] = substr($s_target_string , 0 , $i_range_len);
                    $a_s_area[2] = substr($s_target_string , $i_range_len);
                }
            }
            else{
                if( (($i_start_pos + $i_range_len) >= $i_target_len) or ($i_range_len === 0) ){
                    //ABCDEFGH
                    //01234567
                    //6:2
                    $a_s_area[0] = substr($s_target_string , 0 , $i_start_pos);
                    $a_s_area[1] = substr($s_target_string , $i_start_pos);
                }
                else{
                    //ABCDEFGH
                    //01234567
                    //2:3
                    $a_s_area[0] = substr($s_target_string , 0 , $i_start_pos);
                    $a_s_area[1] = substr($s_target_string , $i_start_pos , $i_range_len);
                    $a_s_area[2] = substr($s_target_string , ($i_start_pos + $i_range_len));
                }
            }
        }
        return $a_s_area;
    }
    /**
     * StrReverse処理
     *
     * @param string $s_target_string 対象文字列
     * @return string
     */
    private function _cmdStrReverse($s_target_string)
    {
        //対象領域確定
        $a_s_area = $this->_divString($s_target_string);
        if( strlen($a_s_area[1]) > 1 ){
            //対象領域が2桁以上の場合
            $a_s_area[1] = strrev($a_s_area[1]);
        }
        $s_converted = $a_s_area[0] . $a_s_area[1] . $a_s_area[2];
        return $s_converted;
    }
    /**
     * CharCycle処理
     *
     * @param string $s_target_string 対象文字列
     * @param boolean $b_map Map使用
     * @return string
     */
    private function _cmdCharCycle($s_target_string , $b_map)
    {
        //方向
        if( (($this->_b_decrypt_mode === false) and ( $this->_a_recipe_inf[$this->_i_current_recipe][1] === 'L' )) or
            (($this->_b_decrypt_mode === true) and ( $this->_a_recipe_inf[$this->_i_current_recipe][1] === 'R' )) ){
            $s_direct = 'L';
        }
        else{
            $s_direct = 'R';
        }
        //文字数
        $i_num = intval($this->_a_recipe_inf[$this->_i_current_recipe][2]);
        //対象領域確定
        $a_s_area = $this->_divString($s_target_string);
        $i_len = strlen($a_s_area[1]);
        $s_cycled = '';
        for($i_ss = 0 ; $i_ss < $i_len ; $i_ss++){
            $s_c = $a_s_area[1][$i_ss];
            if( $b_map === true ){
                $s_c = $this->_getCycleCharByMap($s_direct , $i_num , $s_c);
            }
            else{
                $s_c = $this->_getCycleCharByAscii($s_direct , $i_num , $s_c);
            }
            $s_cycled = $s_cycled . $s_c;
        }
        $s_converted = $a_s_area[0] . $s_cycled . $a_s_area[2];
        return $s_converted;
    }
    /**
     * CharReverse処理
     *
     * @param string $s_target_string 対象文字列
     * @param boolean $b_map Map使用
     * @return string
     */
    private function _cmdCharReverse($s_target_string , $b_map)
    {
        //対象領域確定
        $a_s_area = $this->_divString($s_target_string);
        $i_len = strlen($a_s_area[1]);
        $s_reversed = '';
        for($i_ss = 0 ; $i_ss < $i_len ; $i_ss++){
            $s_c = $a_s_area[1][$i_ss];
            if( $b_map === true ){
                $s_c = $this->_getReverseCharByMap($s_c);
            }
            else{
                $s_c = $this->_getReverseCharByAscii($s_c);
            }
            $s_reversed = $s_reversed . $s_c;
        }
        $s_converted = $a_s_area[0] . $s_reversed . $a_s_area[2];
        return $s_converted;
    }
    /**
     * マップシフト文字取得
     *
     * @param string $s_direct 方向
     * @param int $i_num 文字数
     * @param string $s_c 対象文字
     * @return string
     */
    private function _getCycleCharByMap($s_direct , $i_num , $s_c)
    {
        $i_pos = strpos($this->_s_map_str , $s_c);
        if( $s_direct === 'L'){
            $i_pos = $i_pos - $i_num;
        }
        else{
            $i_pos = $i_pos + $i_num;
        }
        if( $i_pos < 0 ) $i_pos = $i_pos + self::MAP_CHAR_NUM;
        if( $i_pos >= self::MAP_CHAR_NUM ) $i_pos = $i_pos - self::MAP_CHAR_NUM;
        $s_cycle_char = $this->_s_map_str[$i_pos];
        return $s_cycle_char;
    }
    /**
     * ASCIIシフト文字取得
     *
     * @param string $s_direct 方向
     * @param int $i_num 文字数
     * @param string $s_c 対象文字
     * @return string
     */
    private function _getCycleCharByAscii($s_direct , $i_num , $s_c)
    {
        $i_pos = strpos($this->_ASCII_STR , $s_c);
        if( $s_direct === 'L'){
            $i_pos = $i_pos - $i_num;
        }
        else{
            $i_pos = $i_pos + $i_num;
        }
        if( $i_pos < 0 ) $i_pos = $i_pos + self::MAP_CHAR_NUM;
        if( $i_pos >= self::MAP_CHAR_NUM ) $i_pos = $i_pos - self::MAP_CHAR_NUM;
        $s_cycle_char = $this->_ASCII_STR[$i_pos];
        return $s_cycle_char;
    }
    /**
     * マップ反転文字取得
     *
     * @param string $s_c 対象文字
     * @return string
     */
    private function _getReverseCharByMap($s_c)
    {
        $i_pos = strpos($this->_s_map_str , $s_c);
        $i_pos = (self::MAP_CHAR_NUM - 1) - $i_pos;
        $s_reverse_char = $this->_s_map_str[$i_pos];
        return $s_reverse_char;
    }
    /**
     * ASCII反転文字取得
     *
     * @param string $s_c 対象文字
     * @return string
     */
    private function _getReverseCharByAscii($s_c)
    {
        $i_pos = strpos($this->_ASCII_STR , $s_c);
        $i_pos = (self::MAP_CHAR_NUM - 1) - $i_pos;
        $s_reverse_char = $this->_ASCII_STR[$i_pos];
        return $s_reverse_char;
    }
    /**
     * ランダム文字列生成
     *
     * @param int $i_len 生成する長さ
     * @return string
     */
    private function _genRandomString($i_len)
    {
        $s_random_str = '';
        $s_base_str = str_shuffle($this->_ASCII_STR);
        $i_rem = $i_len % self::MAP_CHAR_NUM;
        if( $i_rem > 0 ){
            $s_random_str = $s_random_str . substr($s_base_str , 0 , $i_rem);
        }
        $i_div = floor($i_len / self::MAP_CHAR_NUM);
        if( $i_div > 0 ){
            $s_random_str = $s_random_str . str_shuffle(str_repeat($s_base_str , $i_div));
        }
        return $s_random_str;
    }
    /**
     * 固定文字列生成
     *
     * @param int $i_len 生成する長さ
     * @return string
     */
    private function _genFixedString($i_len)
    {
        $base_str = $this->_s_extend_str;
        $i_shift = ($i_len % 10);
        //文字列の長さを10で割った余りを左方向に循環させる
        if( $i_shift > 0 ) $base_str = substr($base_str , $i_shift) . substr($base_str , 0 , $i_shift);
        //文字列の長さを10で割った余りが2038なら反転させる
        switch($i_shift){
            case 2:
            case 0:
            case 3:
            case 8:
                $base_str = strrev($base_str);
                break;
        }
        $s_fixed_str = '';
        $i_div = floor($i_len / self::MAP_CHAR_NUM);
        if( $i_div > 0 ) $s_fixed_str = str_repeat($base_str , $i_div);
        $i_rem = $i_len % (self::MAP_CHAR_NUM);
        if( $i_rem > 0 ) $s_fixed_str = substr($base_str , 0 , $i_rem) . $s_fixed_str;
        return $s_fixed_str;
    }
    /**
     * デバッグ情報設定
     *
     * @param string $s_msg 出力メッセージ
     * @param boolean $b_recipe_set レシピ情報設定フラグ
     * @param boolean $b_end 終了フラグ
     * @return void
     */
    private function _setDebugInf($s_msg , $b_recipe_set = false , $b_end = false)
    {
        $current_time = $this->_getCurrentTime4Debug();
        if( $current_time === false ){
            $s_proc_time = '';
        }
        else{
            if( $this->_f_debug_last_time === false ){
                $s_proc_time = '';
            }
            elseif( $b_end === true ){
                $s_proc_time = sprintf('total_proc_time=%.6f' , ($current_time - $this->_f_debug_start_time));
            }
            else{
                $s_proc_time = sprintf('proc_time=%.6f ' , ($current_time - $this->_f_debug_last_time));
                $this->_f_debug_last_time = $current_time;
            }
        }
        $this->_f_debug_last_time = $current_time;
        if( $b_recipe_set === true ){
            $s_out_msg = '';
            for($i_ss = 0 ; $i_ss < count($this->_a_recipe_inf[$this->_i_current_recipe]) ; $i_ss++){
                if( $i_ss > 0 ) $s_out_msg = $s_out_msg . ':';
                $s_out_msg = $s_out_msg . $this->_a_recipe_inf[$this->_i_current_recipe][$i_ss];
            }
            $s_out_msg = $s_msg . '(' . $s_out_msg . ')';
        }
        else{
            $s_out_msg = $s_msg;
        }
        $i_ss = count($this->_a_debug_inf);
        $this->_a_debug_inf[$i_ss] = $s_proc_time . $s_out_msg;
        return;
    }
    /**
     * デバッグ用カレントタイム取得
     *
     * @param boolean $b_start 開始時刻取得
     * @return float
     */
    private function _getCurrentTime4Debug($b_start = false)
    {
        if( $b_start === true ){
            if( function_exists('microtime') === false ) return false;
            $this->_f_debug_last_time = false;
        }
        else{
            if( $this->_f_debug_start_time === false ) return false;
        }
        return microtime(true);
    }
    /**
     * マスク対象判定
     *
     * @param  string $s_recipe レシピ名
     * @return boolean
     */
    private function _isMaskTarget($s_recipe)
    {
        if( $this->_i_mask_type === 0 ) return false;
        //return $this->_maskFuncIsLevelTarget($s_recipe);
        $s_file_name = $this->_s_mask_rule_file_dir . $this->_s_mask_rule_file;
        //ファイルステータスチェック
        $i_ret = $this->_checkFileStatus($s_file_name , true);
        switch($i_ret){
            case 0:
                break;
            case 1:
                //ファイル未存在
                throw new Exception('Mask file(' . $s_file_name . ') not exists');
                break;
            default:
                //その他ステータスエラー
                throw new Exception('Mask file(' . $s_file_name . ') status error(' . $i_ret . ')');
        }
        //マスク定義ファイルの有効行をデリミタ:でexplode後、配列にセット
        $r_fp = fopen($s_file_name , 'r');
        if( $r_fp === false ) throw new Exception('Mask file open error');
        $s_ptn = '/^[^0-9a-zA-Z\:\_\-\+\*\.\/]+$/';
        $this->_a_mask_rule_file = array();
        $i_line = 0;
        while(true){
            $s_line = fgets($r_fp);
            if( $s_line !== false ){
                $s_line = trim($s_line);
                if( $s_line !== '' and $s_line[0] !== '#' ){
                    //使用文字チェック(英数字と:以外の文字種別はエラーとする)
                    if( preg_match($s_ptn, $s_line) !== 0 ){
                        fclose($r_fp);
                        throw new Exception('Invalid Mask(char valid line=' . ($i_line + 1) . ')');
                    }
                    //配列セット
                    $this->_a_mask_rule_file[$i_line] = explode(':' , $s_line);
                    $i_line++;
                }
            }
            if( feof($r_fp) === true ) break;
        }
        fclose($r_fp);
        //マスク定義ファイル内容チェック
        for($i_ss = 0 ; $i_ss < $i_line ; $i_ss++){
            if( $s_recipe === $this->_a_mask_rule_file[$i_ss][0] ){
                if( $this->_i_mask_rule_index !== -1 ) throw new Exception('Invalid Mask(dupulication valid line=' . ($i_ss + 1) . '>)');
                $this->_i_mask_rule_index = $i_ss;
            }
        }
        //未定義はマスク対象外
        if( $this->_i_mask_rule_index === -1 ){
            if( $this->_b_strict_mask === true ) throw new Exception('Invalid Mask(target recipe not defined.)');
            return false;
        }
        switch(true){
            case (count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) < 2):
            case ($this->_i_mask_type > 1) and (count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) !== 4):
            case ($this->_b_param_user_level_var === true) and (count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) !== 4):
            case (count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) > 4):
                throw new Exception('Invalid Mask(field def valid line=' . ($this->_i_mask_rule_index + 1) . ')');
        }
        //第2フィールドチェック
        if( $this->_a_mask_rule_file[$this->_i_mask_rule_index][1] === '' ) throw new Exception('Invalid Mask(2nd field not defined valid line=' . ($this->_i_mask_rule_index + 1) . ')');
        if( strlen($this->_a_mask_rule_file[$this->_i_mask_rule_index][1]) > self::MASK_MAX_LEN ) throw new Exception('Invalid Mask(2nd field length valid line=' . ($this->_i_mask_rule_index + 1) . ')');
        //第3フィールドチェック
        if( count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) > 2 ){
            if( strlen($this->_a_mask_rule_file[$this->_i_mask_rule_index][2]) > self::MASK_MAX_LEN ) throw new Exception('Invalid Mask(3rd field length valid line=' . ($this->_i_mask_rule_index + 1) . ')');
        }
        //第4フィールドチェック
        if( count($this->_a_mask_rule_file[$this->_i_mask_rule_index]) === 4 ){
            //第4フィールド
            switch($this->_a_mask_rule_file[$this->_i_mask_rule_index][3]){
                case 'PDEL-MASK-PUBLIC':
                    return true;
                    break;
                case 'PDEL-MASK-PRIVATE':
                    return $this->_b_param_user_level_var;
                    break;
            }
            //第4フィールドの範囲チェック
            if( $this->_i_mask_type === 2 ){
                //数値型
                $i_ret = $this->_checkDefFileNum($this->_a_mask_rule_file[$this->_i_mask_rule_index][3] , $this->_i_mask_user_level_min , $this->_i_mask_user_level_max);
                if( $i_ret !== 0 ) throw new Exception('Invalid Mask(4th field valid line=' . ($this->_i_mask_rule_index + 1) . ' ret=' . $i_ret . ')');
                $i_mask_border_level = intval($this->_a_mask_rule_file[$this->_i_mask_rule_index][3]);
            }
            else{
                //文字列型
                $i_mask_border_level = -1;
                $a_user_level = explode($this->_s_user_level_list_delimiter , $this->_s_user_level_list);
                for( $i_ss=0 ; $i_ss < count($a_user_level);$i_ss++){
                    if( $this->_a_mask_rule_file[$this->_i_mask_rule_index][3] === $a_user_level[$i_ss] ){
                        $i_mask_border_level = $i_ss;
                        break;
                    }
                }
                if( $i_mask_border_level === -1 ) throw new Exception('Invalid Mask(4th field valid line=' . ($this->_i_mask_rule_index + 1) . ')');
            }
            //Empty値の場合は置換済み
            if( is_bool($this->_m_param_user_level) === true ) return $this->_m_param_user_level;
            //権限判定
            if( $this->_b_user_level_asc_order === true ){
                if( $this->_m_param_user_level < $i_mask_border_level ) return true;
            }
            else{
                if( $this->_m_param_user_level > $i_mask_border_level ) return true;
            }
            return false;
        }
        return true;
    }
     /**
     * 権限情報リストチェック
     *
     * @param void
     * @return int
     */
    private function _checkUserLevelListStr()
    {
        $a_user_level = explode($this->_s_user_level_list_delimiter , $this->_s_user_level_list);
        if( count($a_user_level) === 1 ) return 1;
        $a_unique = array_unique($a_user_level);
        if( count($a_user_level) !== count($a_unique) ) return 2;
        $m_order = null;
        for( $i_ss=0 ; $i_ss < count($a_user_level);$i_ss++){
            if( $a_user_level[$i_ss] === '' ) return 3;
            if( $this->_i_mask_type === 3 ){
                if( ctype_digit($a_user_level[$i_ss]) === false ) return 4;
                switch($i_ss){
                    case 0:
                        break;
                    case 1:
                        $b_asc = (intval($a_user_level[1]) > intval($a_user_level[0])) ? true : false;
                        break;
                    default:
                        switch(true){
                            case (intval($a_user_level[$i_ss]) > intval($a_user_level[$i_ss - 1]) and $b_asc === false):
                            case (intval($a_user_level[$i_ss]) < intval($a_user_level[$i_ss - 1]) and $b_asc === true):
                                return 5;
                        }
                }
            }
        }
        return 0;
    }
    /**
     * 数値型ユーザー権限レベル情報設定
     *
     * @param int $i_mask_user_level_min
     * @param int $i_mask_user_level_max
     * @return void
     */
    protected function ___setUserLevelNumber($i_mask_user_level_min , $i_mask_user_level_max)
    {
        if( $this->_i_mask_user_level_min !== null ) $this->_die(__FILE__ . '(' . __LINE__ . ') UL_MIN not PDEL-NULL');
        if( $this->_i_mask_user_level_max !== null ) $this->_die(__FILE__ . '(' . __LINE__ . ') UL_MIN not PDEL-NULL');
        if( is_int($i_mask_user_level_min) === false ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid 1st param(not integer)');
        if( is_int($i_mask_user_level_max) === false ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid 2nd param(not integer)');
        if( $i_mask_user_level_min >= $i_mask_user_level_max ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid param relation');
        $this->_i_mask_user_level_min = $i_mask_user_level_min;
        $this->_i_mask_user_level_max = $i_mask_user_level_max;
        return;
    }
    /**
     * 文字列型ユーザー権限レベル情報設定
     *
     * @param string $s_user_level_string
     * @return void
     */
    protected function ___setUserLevelString($s_user_level_string)
    {
        if( $this->_s_user_level_list !== 'PDEL-NULL' ) $this->_die(__FILE__ . '(' . __LINE__ . ') UL_LIST not PDEL-NULL');
        if( is_string($s_user_level_string) === false ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid param(not string)');
        if( strpos($s_user_level_string , chr(0)) !== false ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid param(null exists)');
        $this->_s_user_level_list = $s_user_level_string;
        $i_ret = $this->_checkUserLevelListStr();
        if( $i_ret !== 0 ) $this->_die(__FILE__ . '(' . __LINE__ . ') invalid param(user_level_string<ret=' . $i_ret . ')');
        return;
    }
    /**
     * 認証情報総レングス取得
     *
     * @param void
     * @return int
     */
    private function _getAuthInfTotalLen()
    {
        //認証情報レングス
        $i_auth_inf_total_len = $this->_a_auth_mode_len[$this->_i_default_auth_mode];
        if( $this->_b_limit_by_encrypt_param !== null ) $i_auth_inf_total_len += 14;
        if( $this->_i_auth_inf_c_mode !== 0 ) $i_auth_inf_total_len += $this->_a_auth_mode_len[$this->_i_auth_inf_c_mode];
        return $i_auth_inf_total_len;
    }
   /**
     * 日時取得
     *
     * @param int      $i_add       基準日時に加算する秒数
     * @return string
     */
    private function _getDateTime($i_add = 0)
    {
        try{
            if( $this->_b_date_time === true ){
                //$o_datetime = new DateTime('now');
                $o_datetime = new DateTime('now' , new DateTimeZone('UTC'));
                if( $i_add !== 0 ){
                    $s_interval = '+' . $i_add . ' seconds';
                    if( $o_datetime->modify($s_interval) === false ) return '';
                }
                $s_result = $o_datetime->format('YmdHis');
            }
            else{
                $i_datetime = time();
                $i_datetime = $i_datetime - $this->_i_time_difference;
                if( $i_add !== 0 ) $i_datetime = $i_datetime + $i_add;
                $s_result = date('YmdHis' , $i_datetime);
            }
            return $s_result;
        } catch (Exception $exc) {
            $s_msg = 'date cal error.' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            throw new Exception($s_msg);
        }
    }
    /**
     * 日時文字列チェック
     *
     * @param string $s_t           チェック対象日時文字列
     * @return int   0:正常 -1～:日時以外
     * @return int
     */
    private function _checkDateTime($s_t)
    {
        if( strlen($s_t) !== 14 ) return -1;
        if( ctype_digit($s_t) === false ) return -2;
        $i_y = intval(substr($s_t , 0 , 4));
        $i_m = intval(substr($s_t , 4 , 2));
        $i_d = intval(substr($s_t , 6 , 2));
        if( checkdate($i_m , $i_d , $i_y) === false ) return -3;
        if( intval(substr($s_t , 8 , 2)) > 23 ) return -4;
        if( intval(substr($s_t , 10 , 2)) > 59 ) return -5;
        if( intval(substr($s_t , 12 , 2)) < 60 ) return 0;
        if( ($this->_b_limit_by_encrypt_param === false) and ($this->_b_decrypt_mode === true) and (intval(substr($s_t , 12 , 2)) === 60) ) return 0;
        return -6;
    }
    /**
     * 日時変換(ローカル→GMT)
     *
     * @param string   $s_datetime 変換対象日時文字列
     * @return string
     */
    private function _local2GMT($s_datetime)
    {
        if( $this->_b_date_time === true ){
            $s_format = substr($s_datetime , 0 , 4) . '-' . substr($s_datetime , 4 , 2) . '-' . substr($s_datetime , 6 , 2) . ' ' .
                        substr($s_datetime , 8 , 2) . ':' . substr($s_datetime , 10 , 2) . ':' . substr($s_datetime , 12 , 2);
            $o_date = new DateTime($s_format);
            $s_interval = ($this->_i_time_difference > 0) ? '-' : '+';
            $s_interval = $s_interval . $this->_i_time_difference . ' seconds';
            if( $o_date->modify($s_interval) === false ) throw new Exception('date cal error');
            return($o_date->format('YmdHis'));
        }
        else{
            $i_time = mktime(intval(substr($s_datetime , 8 , 2)) , intval(substr($s_datetime , 10 , 2)) , substr($s_datetime , 12 , 2) ,
                             intval(substr($s_datetime , 4 , 2)) , intval(substr($s_datetime , 6 , 2)) , substr($s_datetime , 0 , 4) );
            $i_time = $i_time - $this->_i_time_difference;
            return (date('YmdHis' , $i_time));
        }
    }
    /**
     * 日時変換(GMT→ローカル)
     *
     * @param string   $s_datetime 変換対象日時文字列
     * @return string
     */
    private function _GMT2local($s_datetime)
    {
        if( $this->_b_date_time === true ){
            $s_format = substr($s_datetime , 0 , 4) . '-' . substr($s_datetime , 4 , 2) . '-' . substr($s_datetime , 6 , 2) . ' ' .
                        substr($s_datetime , 8 , 2) . ':' . substr($s_datetime , 10 , 2) . ':' . substr($s_datetime , 12 , 2);
            $o_date = new DateTime($s_format);
            $s_interval = ($this->_i_time_difference > 0) ? '+' : '-';
            $s_interval = $s_interval . $this->_i_time_difference . ' seconds';
            if( $o_date->modify($s_interval) === false ) throw new Exception('date cal error');
            return($o_date->format('YmdHis'));
        }
        else{
            $i_time = mktime(intval(substr($s_datetime , 8 , 2)) , intval(substr($s_datetime , 10 , 2)) , substr($s_datetime , 12 , 2) ,
                             intval(substr($s_datetime , 4 , 2)) , intval(substr($s_datetime , 6 , 2)) , substr($s_datetime , 0 , 4) );
            $i_time = $i_time + $this->_i_time_difference;
            return (date('YmdHis' , $i_time));
        }
    }
    /**
     * セッションハイジャックチェックキー項目チェック
     *
     * @param void
     * @return int
     */
    private function _checkSessionHijackCheckKey()
    {
        //IP
        switch($this->_s_hijack_check_ip){
            case 'ZFIP':
            case 'IP':
                if( array_key_exists('REMOTE_ADDR' , $_SERVER) === false ) return 1;
                if( is_string($_SERVER['REMOTE_ADDR']) === false ) return 2;
                if( strpos($_SERVER['REMOTE_ADDR'] , chr(0)) !== false ) return 3;
                if( $this->_s_hijack_check_ip === 'ZFIP' ){
                    switch(true){
                        case array_key_exists('HTTP_CLIENT_IP' , $_SERVER):
                            if( is_string($_SERVER['HTTP_CLIENT_IP']) === false ) return 4;
                            if( strpos($_SERVER['HTTP_CLIENT_IP'] , chr(0)) !== false ) return 5;
                            $s_hc = $_SERVER['HTTP_CLIENT_IP'] . $_SERVER['REMOTE_ADDR'];
                            break;
                        case array_key_exists('HTTP_X_FORWARDED_FOR' , $_SERVER):
                            if( is_string($_SERVER['HTTP_X_FORWARDED_FOR']) === false ) return 6;
                            if( strpos($_SERVER['HTTP_X_FORWARDED_FOR'] , chr(0)) !== false ) return 7;
                            $s_hc = $_SERVER['HTTP_X_FORWARDED_FOR'] . $_SERVER['REMOTE_ADDR'];
                            break;
                        default:
                            $s_hc = $_SERVER['REMOTE_ADDR'];
                    }
                }
                else{
                    $s_hc = $_SERVER['REMOTE_ADDR'];
                }
                break;
            default:
                $s_hc = '';
        }
        //UA
        if( $this->_b_hijack_check_ua === true ){
            if( array_key_exists('HTTP_USER_AGENT' , $_SERVER) === false ) return 11;
            if( is_string($_SERVER['HTTP_USER_AGENT']) === false ) return 12;
            if( strpos($_SERVER['HTTP_USER_AGENT'] , chr(0)) !== false ) return 13;
            $s_hc = $s_hc . $_SERVER['HTTP_USER_AGENT'];
        }
        //LANG
        if( $this->_b_hijack_check_lang === true ){
            if( array_key_exists('HTTP_ACCEPT_LANGUAGE' , $_SERVER) === false ) return 21;
            if( is_string($_SERVER['HTTP_ACCEPT_LANGUAGE']) === false ) return 22;
            if( strpos($_SERVER['HTTP_ACCEPT_LANGUAGE'] , chr(0)) !== false ) return 23;
            $s_hc = $s_hc . $_SERVER['HTTP_ACCEPT_LANGUAGE'];
            if( array_key_exists('HTTP_ACCEPT_CHARSET' , $_SERVER) === true ){
                if( is_string($_SERVER['HTTP_ACCEPT_CHARSET']) === false ) return 24;
                if( strpos($_SERVER['HTTP_ACCEPT_CHARSET'] , chr(0)) !== false ) return 25;
                $s_hc = $s_hc . $_SERVER['HTTP_ACCEPT_CHARSET'];
            }
        }
        //任意情報
        if( $this->_s_user_key_c !== '' ) $s_hc = $s_hc . $this->_s_user_key_c;
        //最大長チェック
        if( strlen($s_hc) > self::SESSION_HIJACK_CHECK_KEY_MAX_LEN ) return 31;
        //ハッシュ値取得
        switch($this->_i_auth_inf_c_mode){
            case 2:
            case 5:
                $this->_s_auth_inf_c = sprintf('%08x' , crc32($s_hc));
                break;
            case 3:
            case 6:
                $this->_s_auth_inf_c = md5($s_hc);
                break;
            default:
                $this->_s_auth_inf_c = sha1($s_hc);
        }
        //レングス情報設定
        if( $this->_i_auth_inf_c_mode >= 5 ) $this->_s_auth_inf_c = sprintf('%04x' , strlen($s_hc)) . $this->_s_auth_inf_c;
        return 0;
    }
    /**
     * レシピを解析し「暗号化後のバイト数」「Extend使用時の暗号化可能最大バイト数」を算出する。
     *
     * @param string $s_recipe_name レシピ名
     * @return array レシピ情報
     */
    public function parseRecipe($s_recipe_name , $opt1 = false , $opt2 = false)
    {
        if( $this->_b_debug_mode === true ) $this->_a_debug_inf = array();
        $a_ret = array();
        $a_ret['status'] = false;
        $a_ret['encryptable_max_size'] = 0;
        $a_ret['after_size'] = 0;
        $a_ret['detail'] = '';
        try {
            $this->_b_decrypt_mode = false;
            $s_target_string = 'a';
            $b_status = $this->_checkParam($s_target_string , $s_recipe_name , $opt1 , $opt2);
            $i_add_total = 0;
            if( $b_status === true ){
                $a_ret['status'] = true;
                $base_auth = $this->_getAuthInfTotalLen();
                for($this->_i_current_recipe = $this->_i_cnv_cmd_start ; $this->_i_current_recipe < $this->_i_recipe_max ; $this->_i_current_recipe++){
                    if( $this->_a_recipe_inf[$this->_i_current_recipe][0] === 'Add' ) $i_add_total += intval($this->_a_recipe_inf[$this->_i_current_recipe][1]);
                }
                if( $this->_a_recipe_inf[$this->_i_cnv_cmd_start][0] === 'Extend' ){
                    $a_ret['encryptable_max_size'] = floor((intval($this->_a_recipe_inf[$this->_i_cnv_cmd_start][1]) - $base_auth) / 2);
                    $a_ret['after_size'] = intval($this->_a_recipe_inf[$this->_i_cnv_cmd_start][1]) + $i_add_total;
                }
                else{
                    $a_ret['encryptable_max_size'] = self::ENCRYPT_MAX_SIZE;
                    $a_ret['after_size'] = '{(PDEL-BEFORE-DATA-SIZE_X_2)+}' . ($base_auth + $i_add_total);
                }
            }
            else{
                $a_ret['detail'] = $this->_s_error_inf;
            }
            $this->_eraseVar();
            return $a_ret;
        } catch (Exception $exc) {
            $a_ret['detail'] = 'Z:### ' . __FUNCTION__ . '(' . __LINE__ . ')' . basename($exc->getFile()) . '(' . $exc->getLine() . ')' . $exc->getMessage();
            $this->_eraseVar();
            return $a_ret;
        }
    }
    /**
     * エラー処理
     *
     * @param array $a_ret OutputValueArray
     * @param string $s_recipe_name
     * @return void
     */
    protected function ___error_proc($a_ret , $s_recipe_name)
    {
        /*
         * 独自のエラー処理を実装される場合、このメソッドをオーバーライドしてください
         */
    }
    /**
     * 変数消去
     *
     * @param void
     * @return void
     */
    private function _eraseVar()
    {
        //重要情報を格納している変数がcoreダンプに出力されるのを防ぐため念のため消去する
        if( isset($this->_s_map_str) === true ){
            $this->_eraseString($this->_s_map_str);
            unset($this->_s_map_str);
        }
        if( isset($this->_s_extend_str) === true ){
            $this->_eraseString($this->_s_extend_str);
            unset($this->_s_extend_str);
        }
        if( isset($this->_s_map_fixed_ss) === true ){
            $this->_eraseString($this->_s_map_fixed_ss);
            unset($this->_s_map_fixed_ss);
        }
        if( isset($this->_s_result) === true ){
            $this->_eraseString($this->_s_result);
            unset($this->_s_result);
        }
        if( (isset($this->_a_recipe_inf) === true) and (is_array($this->_a_recipe_inf) === true) ){
            $i1 = count($this->_a_recipe_inf) - 1;
            for(;$i1 >= 0;$i1--){
                if( is_array($this->_a_recipe_inf[$i1]) === true ){
                    $i2 = count($this->_a_recipe_inf[$i1]) - 1;
                    for(;$i2 >= 0;$i2--){
                        $this->_eraseString($this->_a_recipe_inf[$i1][$i2]);
                        unset($this->_a_recipe_inf[$i1][$i2]);
                    }
                }
                unset($this->_a_recipe_inf[$i1]);
            }
            unset($this->_a_recipe_inf);
        }
        if( (isset($this->_a_mask_rule_file) === true) and (is_array($this->_a_mask_rule_file) === true) ){
            $i1 = count($this->_a_mask_rule_file) - 1;
            for(;$i1 >= 0;$i1--){
                if( is_array($this->_a_mask_rule_file[$i1]) === true ){
                    $i2 = count($this->_a_mask_rule_file[$i1]) - 1;
                    for(;$i2 >= 0;$i2--){
                        $this->_eraseString($this->_a_mask_rule_file[$i1][$i2]);
                        unset($this->_a_mask_rule_file[$i1][$i2]);
                    }
                }
                unset($this->_a_mask_rule_file[$i1]);
            }
            unset($this->_a_mask_rule_file);
        }
        if( (isset($this->_a_debug_inf) === true) and (is_array($this->_a_debug_inf) === true) ){
            $i1 = count($this->_a_debug_inf) - 1;
            for(;$i1 >= 0;$i1--){
                $this->_eraseString($this->_a_debug_inf[$i1]);
                unset($this->_a_debug_inf[$i1]);
            }
            unset($this->_a_debug_inf);
        }
    }
    /**
     * 文字列消去
     *
     * @param string $s_string 消去対象文字列
     * @return void
     */
    private function _eraseString(&$s_string)
    {
        $i_len = strlen($s_string);
        $s_dummy = str_repeat(chr(0) , $i_len);
        $s_string = $s_dummy;
    }
    /**
     * デストラクタ
     *
     * @param void
     * @return void
     */
    public function __destruct()
    {
    }
    /**
     * ディレクトリ解析
     *
     * @param string  $s_parse_path 解析パス
     * @param boolean $b_file       ファイル区分(解析パスがファイルの場合true、ディレクトリの場合false)
     * @return string               解析後絶対パス。解析エラー時は空文字列
     */
    private function _parsePath($s_parse_path , $b_path_type)
    {
        if( is_string($s_parse_path) === false ) return '';
        $s_path = $s_parse_path;
        if( $s_path === '' ){
            $s_path = dirname(__FILE__);
            if( substr($s_path , -1) !== DIRECTORY_SEPARATOR ) $s_path = $s_path . DIRECTORY_SEPARATOR;
            return $s_path;
        }
        //$s_parse_pathのうち、<>で囲まれた文字列は定数、{}で囲まれた文字列は環境変数で置換する
        while(true){
            $i_start_pos1 = strpos($s_path , '<');
            $i_start_pos2 = strpos($s_path , '{');
            if( ($i_start_pos1 === false) and ($i_start_pos2 === false) ) break;
            if( $i_start_pos1 !== false ){
                //定数置換
                $i_end_pos1 = strpos($s_path , '>');
                if( ($i_end_pos1 === false) or ($i_end_pos1 < ($i_start_pos1 + 1)) ) return '';
                $s_def_val = substr($s_path , ($i_start_pos1 + 1) , ($i_end_pos1 - $i_start_pos1 - 1));
                $s_replace_val = constant($s_def_val);
                if( $s_replace_val === null ) return '';
                $s_def_val = '<' . $s_def_val . '>';
                $s_path = str_replace($s_def_val , $s_replace_val , $s_path);
            }
            if( $i_start_pos2 !== false ){
                //環境変数置換
                $i_end_pos2 = strpos($s_path , '}');
                if( ($i_end_pos2 === false) or ($i_end_pos2 < ($i_start_pos2 + 1)) ) return '';
                $s_def_val = substr($s_path , ($i_start_pos2 + 1) , ($i_end_pos2 - $i_start_pos2 - 1));
                $s_replace_val = getenv($s_def_val);
                if( $s_replace_val === false ) return '';
                $s_def_val = '{' . $s_def_val . '}';
                $s_path = str_replace($s_def_val , $s_replace_val , $s_path);
            }
        }
        if( $s_path[0] === '.' ){
            $s_path = dirname(__FILE__) . DIRECTORY_SEPARATOR . $s_path;
        }
        $s_path = realpath($s_path);
        if( $s_path === false ) return '';
        if( file_exists($s_path) === false ) return '';
        if( $b_path_type === true ){
            if( is_file($s_path) === false ) return '';
        }
        else{
            if( is_dir($s_path) === false ) return '';
            if( substr($s_path , -1) !== DIRECTORY_SEPARATOR ) $s_path = $s_path . DIRECTORY_SEPARATOR;
        }
        return $s_path;
    }
    /**
     * 異常終了
     *
     * @param string
     * @return void
     */
    private function _die($msg)
    {
        die($msg);
    }
}